@@ -1,12 +1,29 @@
 """Device automation helpers for toggle entity."""
 from __future__ import annotations
 
-from typing import Any
-
 import voluptuous as vol
 
-from homeassistant.components.automation import AutomationActionType
-from homeassistant.components.device_automation.const import (
+from homeassistant.components.homeassistant.triggers import state as state_trigger
+from homeassistant.const import (
+    ATTR_ENTITY_ID,
+    CONF_CONDITION,
+    CONF_ENTITY_ID,
+    CONF_FOR,
+    CONF_PLATFORM,
+    CONF_STATE,
+    CONF_TYPE,
+)
+from homeassistant.core import CALLBACK_TYPE, Context, HomeAssistant, callback
+from homeassistant.helpers import (
+    condition,
+    config_validation as cv,
+    entity_registry as er,
+)
+from homeassistant.helpers.trigger import TriggerActionType, TriggerInfo
+from homeassistant.helpers.typing import ConfigType, TemplateVarsType
+
+from . import DEVICE_TRIGGER_BASE_SCHEMA, entity
+from .const import (
     CONF_IS_OFF,
     CONF_IS_ON,
     CONF_TOGGLE,
@@ -15,21 +32,6 @@ from homeassistant.components.device_automation.const import (
     CONF_TURNED_OFF,
     CONF_TURNED_ON,
 )
-from homeassistant.components.homeassistant.triggers import state as state_trigger
-from homeassistant.const import (
-    ATTR_ENTITY_ID,
-    CONF_CONDITION,
-    CONF_ENTITY_ID,
-    CONF_FOR,
-    CONF_PLATFORM,
-    CONF_TYPE,
-)
-from homeassistant.core import CALLBACK_TYPE, Context, HomeAssistant, callback
-from homeassistant.helpers import condition, config_validation as cv
-from homeassistant.helpers.entity_registry import async_entries_for_device
-from homeassistant.helpers.typing import ConfigType, TemplateVarsType
-
-from . import TRIGGER_BASE_SCHEMA
 
 # mypy: allow-untyped-calls, allow-untyped-defs
 
@@ -91,7 +93,7 @@ CONDITION_SCHEMA = cv.DEVICE_CONDITION_BASE_SCHEMA.extend(
     }
 )
 
-TRIGGER_SCHEMA = TRIGGER_BASE_SCHEMA.extend(
+_TOGGLE_TRIGGER_SCHEMA = DEVICE_TRIGGER_BASE_SCHEMA.extend(
     {
         vol.Required(CONF_ENTITY_ID): cv.entity_id,
         vol.Required(CONF_TYPE): vol.In([CONF_TURNED_OFF, CONF_TURNED_ON]),
@@ -99,12 +101,14 @@ TRIGGER_SCHEMA = TRIGGER_BASE_SCHEMA.extend(
     }
 )
 
+TRIGGER_SCHEMA = vol.Any(entity.TRIGGER_SCHEMA, _TOGGLE_TRIGGER_SCHEMA)
+
 
 async def async_call_action_from_config(
     hass: HomeAssistant,
     config: ConfigType,
     variables: TemplateVarsType,
-    context: Context,
+    context: Context | None,
     domain: str,
 ) -> None:
     """Change state based on configuration."""
@@ -124,33 +128,38 @@ async def async_call_action_from_config(
 
 
 @callback
-def async_condition_from_config(config: ConfigType) -> condition.ConditionCheckerType:
+def async_condition_from_config(
+    hass: HomeAssistant, config: ConfigType
+) -> condition.ConditionCheckerType:
     """Evaluate state based on configuration."""
-    condition_type = config[CONF_TYPE]
-    if condition_type == CONF_IS_ON:
+    if config[CONF_TYPE] == CONF_IS_ON:
         stat = "on"
     else:
         stat = "off"
     state_config = {
-        condition.CONF_CONDITION: "state",
-        condition.CONF_ENTITY_ID: config[CONF_ENTITY_ID],
-        condition.CONF_STATE: stat,
+        CONF_CONDITION: "state",
+        CONF_ENTITY_ID: config[CONF_ENTITY_ID],
+        CONF_STATE: stat,
     }
     if CONF_FOR in config:
         state_config[CONF_FOR] = config[CONF_FOR]
 
+    state_config = cv.STATE_CONDITION_SCHEMA(state_config)
+    state_config = condition.state_validate_config(hass, state_config)
     return condition.state_from_config(state_config)
 
 
 async def async_attach_trigger(
     hass: HomeAssistant,
     config: ConfigType,
-    action: AutomationActionType,
-    automation_info: dict,
+    action: TriggerActionType,
+    trigger_info: TriggerInfo,
 ) -> CALLBACK_TYPE:
     """Listen for state changes based on configuration."""
-    trigger_type = config[CONF_TYPE]
-    if trigger_type == CONF_TURNED_ON:
+    if config[CONF_TYPE] not in [CONF_TURNED_ON, CONF_TURNED_OFF]:
+        return await entity.async_attach_trigger(hass, config, action, trigger_info)
+
+    if config[CONF_TYPE] == CONF_TURNED_ON:
         to_state = "on"
     else:
         to_state = "off"
@@ -162,22 +171,25 @@ async def async_attach_trigger(
     if CONF_FOR in config:
         state_config[CONF_FOR] = config[CONF_FOR]
 
-    state_config = state_trigger.TRIGGER_SCHEMA(state_config)
+    state_config = await state_trigger.async_validate_trigger_config(hass, state_config)
     return await state_trigger.async_attach_trigger(
-        hass, state_config, action, automation_info, platform_type="device"
+        hass, state_config, action, trigger_info, platform_type="device"
     )
 
 
 async def _async_get_automations(
-    hass: HomeAssistant, device_id: str, automation_templates: list[dict], domain: str
-) -> list[dict]:
+    hass: HomeAssistant,
+    device_id: str,
+    automation_templates: list[dict[str, str]],
+    domain: str,
+) -> list[dict[str, str]]:
     """List device automations."""
-    automations: list[dict[str, Any]] = []
-    entity_registry = await hass.helpers.entity_registry.async_get_registry()
+    automations: list[dict[str, str]] = []
+    entity_registry = er.async_get(hass)
 
     entries = [
         entry
-        for entry in async_entries_for_device(entity_registry, device_id)
+        for entry in er.async_entries_for_device(entity_registry, device_id)
         if entry.domain == domain
     ]
 
@@ -197,7 +209,7 @@ async def _async_get_automations(
 
 async def async_get_actions(
     hass: HomeAssistant, device_id: str, domain: str
-) -> list[dict]:
+) -> list[dict[str, str]]:
     """List device actions."""
     return await _async_get_automations(hass, device_id, ENTITY_ACTIONS, domain)
 
@@ -211,12 +223,18 @@ async def async_get_conditions(
 
 async def async_get_triggers(
     hass: HomeAssistant, device_id: str, domain: str
-) -> list[dict]:
+) -> list[dict[str, str]]:
     """List device triggers."""
-    return await _async_get_automations(hass, device_id, ENTITY_TRIGGERS, domain)
+    triggers = await entity.async_get_triggers(hass, device_id, domain)
+    triggers.extend(
+        await _async_get_automations(hass, device_id, ENTITY_TRIGGERS, domain)
+    )
+    return triggers
 
 
-async def async_get_condition_capabilities(hass: HomeAssistant, config: dict) -> dict:
+async def async_get_condition_capabilities(
+    hass: HomeAssistant, config: ConfigType
+) -> dict[str, vol.Schema]:
     """List condition capabilities."""
     return {
         "extra_fields": vol.Schema(
@@ -225,8 +243,13 @@ async def async_get_condition_capabilities(hass: HomeAssistant, config: dict) ->
     }
 
 
-async def async_get_trigger_capabilities(hass: HomeAssistant, config: dict) -> dict:
+async def async_get_trigger_capabilities(
+    hass: HomeAssistant, config: ConfigType
+) -> dict[str, vol.Schema]:
     """List trigger capabilities."""
+    if config[CONF_TYPE] not in [CONF_TURNED_ON, CONF_TURNED_OFF]:
+        return await entity.async_get_trigger_capabilities(hass, config)
+
     return {
         "extra_fields": vol.Schema(
             {vol.Optional(CONF_FOR): cv.positive_time_period_dict}
