@@ -5,12 +5,14 @@ from collections.abc import Iterable
 import csv
 import dataclasses
 from datetime import timedelta
+from enum import IntEnum
 import logging
 import os
 from typing import cast, final
 
 import voluptuous as vol
 
+from homeassistant.backports.enum import StrEnum
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     SERVICE_TOGGLE,
@@ -19,14 +21,16 @@ from homeassistant.const import (
     STATE_ON,
 )
 from homeassistant.core import HomeAssistant, callback
-import homeassistant.helpers.config_validation as cv
+from homeassistant.exceptions import HomeAssistantError
+from homeassistant.helpers import config_validation as cv, entity_registry as er
 from homeassistant.helpers.config_validation import (  # noqa: F401
     PLATFORM_SCHEMA,
     PLATFORM_SCHEMA_BASE,
     make_entity_service_schema,
 )
-from homeassistant.helpers.entity import ToggleEntity
+from homeassistant.helpers.entity import ToggleEntity, ToggleEntityDescription
 from homeassistant.helpers.entity_component import EntityComponent
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.loader import bind_hass
 import homeassistant.util.color as color_util
 
@@ -38,82 +42,149 @@ DATA_PROFILES = "light_profiles"
 
 ENTITY_ID_FORMAT = DOMAIN + ".{}"
 
-# Bitfield of features supported by the light entity
+
+class LightEntityFeature(IntEnum):
+    """Supported features of the light entity."""
+
+    EFFECT = 4
+    FLASH = 8
+    TRANSITION = 32
+
+
+# These SUPPORT_* constants are deprecated as of Home Assistant 2022.5.
+# Please use the LightEntityFeature enum instead.
 SUPPORT_BRIGHTNESS = 1  # Deprecated, replaced by color modes
 SUPPORT_COLOR_TEMP = 2  # Deprecated, replaced by color modes
 SUPPORT_EFFECT = 4
 SUPPORT_FLASH = 8
 SUPPORT_COLOR = 16  # Deprecated, replaced by color modes
 SUPPORT_TRANSITION = 32
-SUPPORT_WHITE_VALUE = 128  # Deprecated, replaced by color modes
 
 # Color mode of the light
 ATTR_COLOR_MODE = "color_mode"
 # List of color modes supported by the light
 ATTR_SUPPORTED_COLOR_MODES = "supported_color_modes"
-# Possible color modes
-COLOR_MODE_UNKNOWN = "unknown"  # Ambiguous color mode
-COLOR_MODE_ONOFF = "onoff"  # Must be the only supported mode
-COLOR_MODE_BRIGHTNESS = "brightness"  # Must be the only supported mode
+
+
+class ColorMode(StrEnum):
+    """Possible light color modes."""
+
+    UNKNOWN = "unknown"  # Ambiguous color mode
+    ONOFF = "onoff"  # Must be the only supported mode
+    BRIGHTNESS = "brightness"  # Must be the only supported mode
+    COLOR_TEMP = "color_temp"
+    HS = "hs"
+    XY = "xy"
+    RGB = "rgb"
+    RGBW = "rgbw"
+    RGBWW = "rgbww"
+    WHITE = "white"  # Must *NOT* be the only supported mode
+
+
+# These COLOR_MODE_* constants are deprecated as of Home Assistant 2022.5.
+# Please use the LightEntityFeature enum instead.
+COLOR_MODE_UNKNOWN = "unknown"
+COLOR_MODE_ONOFF = "onoff"
+COLOR_MODE_BRIGHTNESS = "brightness"
 COLOR_MODE_COLOR_TEMP = "color_temp"
 COLOR_MODE_HS = "hs"
 COLOR_MODE_XY = "xy"
 COLOR_MODE_RGB = "rgb"
 COLOR_MODE_RGBW = "rgbw"
 COLOR_MODE_RGBWW = "rgbww"
+COLOR_MODE_WHITE = "white"
 
 VALID_COLOR_MODES = {
-    COLOR_MODE_ONOFF,
-    COLOR_MODE_BRIGHTNESS,
-    COLOR_MODE_COLOR_TEMP,
-    COLOR_MODE_HS,
-    COLOR_MODE_XY,
-    COLOR_MODE_RGB,
-    COLOR_MODE_RGBW,
-    COLOR_MODE_RGBWW,
+    ColorMode.ONOFF,
+    ColorMode.BRIGHTNESS,
+    ColorMode.COLOR_TEMP,
+    ColorMode.HS,
+    ColorMode.XY,
+    ColorMode.RGB,
+    ColorMode.RGBW,
+    ColorMode.RGBWW,
+    ColorMode.WHITE,
 }
-COLOR_MODES_BRIGHTNESS = VALID_COLOR_MODES - {COLOR_MODE_ONOFF}
+COLOR_MODES_BRIGHTNESS = VALID_COLOR_MODES - {ColorMode.ONOFF}
 COLOR_MODES_COLOR = {
-    COLOR_MODE_HS,
-    COLOR_MODE_RGB,
-    COLOR_MODE_RGBW,
-    COLOR_MODE_RGBWW,
-    COLOR_MODE_XY,
+    ColorMode.HS,
+    ColorMode.RGB,
+    ColorMode.RGBW,
+    ColorMode.RGBWW,
+    ColorMode.XY,
 }
 
 
-def valid_supported_color_modes(color_modes: Iterable[str]) -> set[str]:
+def filter_supported_color_modes(color_modes: Iterable[ColorMode]) -> set[ColorMode]:
+    """Filter the given color modes."""
+    color_modes = set(color_modes)
+    if (
+        not color_modes
+        or ColorMode.UNKNOWN in color_modes
+        or (ColorMode.WHITE in color_modes and not color_supported(color_modes))
+    ):
+        raise HomeAssistantError
+
+    if ColorMode.ONOFF in color_modes and len(color_modes) > 1:
+        color_modes.remove(ColorMode.ONOFF)
+    if ColorMode.BRIGHTNESS in color_modes and len(color_modes) > 1:
+        color_modes.remove(ColorMode.BRIGHTNESS)
+    return color_modes
+
+
+def valid_supported_color_modes(
+    color_modes: Iterable[ColorMode | str],
+) -> set[ColorMode | str]:
     """Validate the given color modes."""
     color_modes = set(color_modes)
     if (
         not color_modes
-        or COLOR_MODE_UNKNOWN in color_modes
-        or (COLOR_MODE_BRIGHTNESS in color_modes and len(color_modes) > 1)
-        or (COLOR_MODE_ONOFF in color_modes and len(color_modes) > 1)
+        or ColorMode.UNKNOWN in color_modes
+        or (ColorMode.BRIGHTNESS in color_modes and len(color_modes) > 1)
+        or (ColorMode.ONOFF in color_modes and len(color_modes) > 1)
+        or (ColorMode.WHITE in color_modes and not color_supported(color_modes))
     ):
         raise vol.Error(f"Invalid supported_color_modes {sorted(color_modes)}")
     return color_modes
 
 
-def brightness_supported(color_modes: Iterable[str] | None) -> bool:
+def brightness_supported(color_modes: Iterable[ColorMode | str] | None) -> bool:
     """Test if brightness is supported."""
     if not color_modes:
         return False
     return any(mode in COLOR_MODES_BRIGHTNESS for mode in color_modes)
 
 
-def color_supported(color_modes: Iterable[str] | None) -> bool:
+def color_supported(color_modes: Iterable[ColorMode | str] | None) -> bool:
     """Test if color is supported."""
     if not color_modes:
         return False
     return any(mode in COLOR_MODES_COLOR for mode in color_modes)
 
 
-def color_temp_supported(color_modes: Iterable[str] | None) -> bool:
+def color_temp_supported(color_modes: Iterable[ColorMode | str] | None) -> bool:
     """Test if color temperature is supported."""
     if not color_modes:
         return False
-    return COLOR_MODE_COLOR_TEMP in color_modes
+    return ColorMode.COLOR_TEMP in color_modes
+
+
+def get_supported_color_modes(hass: HomeAssistant, entity_id: str) -> set | None:
+    """Get supported color modes for a light entity.
+
+    First try the statemachine, then entity registry.
+    This is the equivalent of entity helper get_supported_features.
+    """
+    if state := hass.states.get(entity_id):
+        return state.attributes.get(ATTR_SUPPORTED_COLOR_MODES)
+
+    entity_registry = er.async_get(hass)
+    if not (entry := entity_registry.async_get(entity_id)):
+        raise HomeAssistantError(f"Unknown entity {entity_id}")
+    if not entry.capabilities:
+        return None
+
+    return entry.capabilities.get(ATTR_SUPPORTED_COLOR_MODES)
 
 
 # Float that represents transition time in seconds to make change.
@@ -130,7 +201,7 @@ ATTR_KELVIN = "kelvin"
 ATTR_MIN_MIREDS = "min_mireds"
 ATTR_MAX_MIREDS = "max_mireds"
 ATTR_COLOR_NAME = "color_name"
-ATTR_WHITE_VALUE = "white_value"
+ATTR_WHITE = "white"
 
 # Brightness of the light, 0..255 or percentage
 ATTR_BRIGHTNESS = "brightness"
@@ -175,32 +246,32 @@ LIGHT_TURN_ON_SCHEMA = {
     vol.Exclusive(ATTR_BRIGHTNESS_STEP, ATTR_BRIGHTNESS): VALID_BRIGHTNESS_STEP,
     vol.Exclusive(ATTR_BRIGHTNESS_STEP_PCT, ATTR_BRIGHTNESS): VALID_BRIGHTNESS_STEP_PCT,
     vol.Exclusive(ATTR_COLOR_NAME, COLOR_GROUP): cv.string,
-    vol.Exclusive(ATTR_RGB_COLOR, COLOR_GROUP): vol.All(
-        vol.ExactSequence((cv.byte,) * 3), vol.Coerce(tuple)
-    ),
-    vol.Exclusive(ATTR_RGBW_COLOR, COLOR_GROUP): vol.All(
-        vol.ExactSequence((cv.byte,) * 4), vol.Coerce(tuple)
-    ),
-    vol.Exclusive(ATTR_RGBWW_COLOR, COLOR_GROUP): vol.All(
-        vol.ExactSequence((cv.byte,) * 5), vol.Coerce(tuple)
-    ),
-    vol.Exclusive(ATTR_XY_COLOR, COLOR_GROUP): vol.All(
-        vol.ExactSequence((cv.small_float, cv.small_float)), vol.Coerce(tuple)
+    vol.Exclusive(ATTR_COLOR_TEMP, COLOR_GROUP): vol.All(
+        vol.Coerce(int), vol.Range(min=1)
     ),
+    vol.Exclusive(ATTR_KELVIN, COLOR_GROUP): cv.positive_int,
     vol.Exclusive(ATTR_HS_COLOR, COLOR_GROUP): vol.All(
+        vol.Coerce(tuple),
         vol.ExactSequence(
             (
                 vol.All(vol.Coerce(float), vol.Range(min=0, max=360)),
                 vol.All(vol.Coerce(float), vol.Range(min=0, max=100)),
             )
         ),
-        vol.Coerce(tuple),
     ),
-    vol.Exclusive(ATTR_COLOR_TEMP, COLOR_GROUP): vol.All(
-        vol.Coerce(int), vol.Range(min=1)
+    vol.Exclusive(ATTR_RGB_COLOR, COLOR_GROUP): vol.All(
+        vol.Coerce(tuple), vol.ExactSequence((cv.byte,) * 3)
     ),
-    vol.Exclusive(ATTR_KELVIN, COLOR_GROUP): cv.positive_int,
-    ATTR_WHITE_VALUE: vol.All(vol.Coerce(int), vol.Range(min=0, max=255)),
+    vol.Exclusive(ATTR_RGBW_COLOR, COLOR_GROUP): vol.All(
+        vol.Coerce(tuple), vol.ExactSequence((cv.byte,) * 4)
+    ),
+    vol.Exclusive(ATTR_RGBWW_COLOR, COLOR_GROUP): vol.All(
+        vol.Coerce(tuple), vol.ExactSequence((cv.byte,) * 5)
+    ),
+    vol.Exclusive(ATTR_XY_COLOR, COLOR_GROUP): vol.All(
+        vol.Coerce(tuple), vol.ExactSequence((cv.small_float, cv.small_float))
+    ),
+    vol.Exclusive(ATTR_WHITE, COLOR_GROUP): VALID_BRIGHTNESS,
     ATTR_FLASH: VALID_FLASH,
     ATTR_EFFECT: cv.string,
 }
@@ -229,16 +300,14 @@ def preprocess_turn_on_alternatives(hass, params):
     if ATTR_PROFILE in params:
         hass.data[DATA_PROFILES].apply_profile(params.pop(ATTR_PROFILE), params)
 
-    color_name = params.pop(ATTR_COLOR_NAME, None)
-    if color_name is not None:
+    if (color_name := params.pop(ATTR_COLOR_NAME, None)) is not None:
         try:
             params[ATTR_RGB_COLOR] = color_util.color_name_to_rgb(color_name)
         except ValueError:
             _LOGGER.warning("Got unknown color %s, falling back to white", color_name)
             params[ATTR_RGB_COLOR] = (255, 255, 255)
 
-    kelvin = params.pop(ATTR_KELVIN, None)
-    if kelvin is not None:
+    if (kelvin := params.pop(ATTR_KELVIN, None)) is not None:
         mired = color_util.color_temperature_kelvin_to_mired(kelvin)
         params[ATTR_COLOR_TEMP] = int(mired)
 
@@ -247,12 +316,53 @@ def preprocess_turn_on_alternatives(hass, params):
         params[ATTR_BRIGHTNESS] = round(255 * brightness_pct / 100)
 
 
-def filter_turn_off_params(params):
-    """Filter out params not used in turn off."""
+def filter_turn_off_params(light, params):
+    """Filter out params not used in turn off or not supported by the light."""
+    supported_features = light.supported_features
+
+    if not supported_features & LightEntityFeature.FLASH:
+        params.pop(ATTR_FLASH, None)
+    if not supported_features & LightEntityFeature.TRANSITION:
+        params.pop(ATTR_TRANSITION, None)
+
     return {k: v for k, v in params.items() if k in (ATTR_TRANSITION, ATTR_FLASH)}
 
 
-async def async_setup(hass, config):  # noqa: C901
+def filter_turn_on_params(light, params):
+    """Filter out params not supported by the light."""
+    supported_features = light.supported_features
+
+    if not supported_features & LightEntityFeature.EFFECT:
+        params.pop(ATTR_EFFECT, None)
+    if not supported_features & LightEntityFeature.FLASH:
+        params.pop(ATTR_FLASH, None)
+    if not supported_features & LightEntityFeature.TRANSITION:
+        params.pop(ATTR_TRANSITION, None)
+
+    supported_color_modes = (
+        light._light_internal_supported_color_modes  # pylint:disable=protected-access
+    )
+    if not brightness_supported(supported_color_modes):
+        params.pop(ATTR_BRIGHTNESS, None)
+    if ColorMode.COLOR_TEMP not in supported_color_modes:
+        params.pop(ATTR_COLOR_TEMP, None)
+    if ColorMode.HS not in supported_color_modes:
+        params.pop(ATTR_HS_COLOR, None)
+    if ColorMode.RGB not in supported_color_modes:
+        params.pop(ATTR_RGB_COLOR, None)
+    if ColorMode.RGBW not in supported_color_modes:
+        params.pop(ATTR_RGBW_COLOR, None)
+    if ColorMode.RGBWW not in supported_color_modes:
+        params.pop(ATTR_RGBWW_COLOR, None)
+    if ColorMode.WHITE not in supported_color_modes:
+        params.pop(ATTR_WHITE, None)
+    if ColorMode.XY not in supported_color_modes:
+        params.pop(ATTR_XY_COLOR, None)
+
+    return params
+
+
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:  # noqa: C901
     """Expose light control via state machine and services."""
     component = hass.data[DOMAIN] = EntityComponent(
         _LOGGER, DOMAIN, hass, SCAN_INTERVAL
@@ -302,20 +412,28 @@ async def async_setup(hass, config):  # noqa: C901
         ):
             profiles.apply_default(light.entity_id, light.is_on, params)
 
+        legacy_supported_color_modes = (
+            light._light_internal_supported_color_modes  # pylint: disable=protected-access
+        )
         supported_color_modes = light.supported_color_modes
-        # Backwards compatibility: if an RGBWW color is specified, convert to RGB + W
-        # for legacy lights
-        if ATTR_RGBW_COLOR in params:
-            legacy_supported_color_modes = (
-                light._light_internal_supported_color_modes  # pylint: disable=protected-access
-            )
+
+        # If a color temperature is specified, emulate it if not supported by the light
+        if ATTR_COLOR_TEMP in params:
             if (
-                COLOR_MODE_RGBW in legacy_supported_color_modes
-                and not supported_color_modes
+                supported_color_modes
+                and ColorMode.COLOR_TEMP not in supported_color_modes
+                and ColorMode.RGBWW in supported_color_modes
             ):
-                rgbw_color = params.pop(ATTR_RGBW_COLOR)
-                params[ATTR_RGB_COLOR] = rgbw_color[0:3]
-                params[ATTR_WHITE_VALUE] = rgbw_color[3]
+                color_temp = params.pop(ATTR_COLOR_TEMP)
+                brightness = params.get(ATTR_BRIGHTNESS, light.brightness)
+                params[ATTR_RGBWW_COLOR] = color_util.color_temperature_to_rgbww(
+                    color_temp, brightness, light.min_mireds, light.max_mireds
+                )
+            elif ColorMode.COLOR_TEMP not in legacy_supported_color_modes:
+                color_temp = params.pop(ATTR_COLOR_TEMP)
+                if color_supported(legacy_supported_color_modes):
+                    temp_k = color_util.color_temperature_mired_to_kelvin(color_temp)
+                    params[ATTR_HS_COLOR] = color_util.color_temperature_to_hs(temp_k)
 
         # If a color is specified, convert to the color space supported by the light
         # Backwards compatibility: Fall back to hs color if light.supported_color_modes
@@ -325,55 +443,96 @@ async def async_setup(hass, config):  # noqa: C901
                 params[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
             elif (xy_color := params.pop(ATTR_XY_COLOR, None)) is not None:
                 params[ATTR_HS_COLOR] = color_util.color_xy_to_hs(*xy_color)
-        elif ATTR_HS_COLOR in params and COLOR_MODE_HS not in supported_color_modes:
-            hs_color = params.pop(ATTR_HS_COLOR)
-            if COLOR_MODE_RGB in supported_color_modes:
-                params[ATTR_RGB_COLOR] = color_util.color_hs_to_RGB(*hs_color)
-            elif COLOR_MODE_RGBW in supported_color_modes:
-                rgb_color = color_util.color_hs_to_RGB(*hs_color)
-                params[ATTR_RGBW_COLOR] = color_util.color_rgb_to_rgbw(*rgb_color)
-            elif COLOR_MODE_RGBWW in supported_color_modes:
-                rgb_color = color_util.color_hs_to_RGB(*hs_color)
-                params[ATTR_RGBWW_COLOR] = color_util.color_rgb_to_rgbww(
-                    *rgb_color, light.min_mireds, light.max_mireds
-                )
-            elif COLOR_MODE_XY in supported_color_modes:
-                params[ATTR_XY_COLOR] = color_util.color_hs_to_xy(*hs_color)
-        elif ATTR_RGB_COLOR in params and COLOR_MODE_RGB not in supported_color_modes:
-            rgb_color = params.pop(ATTR_RGB_COLOR)
-            if COLOR_MODE_RGBW in supported_color_modes:
-                params[ATTR_RGBW_COLOR] = color_util.color_rgb_to_rgbw(*rgb_color)
-            elif COLOR_MODE_RGBWW in supported_color_modes:
-                params[ATTR_RGBWW_COLOR] = color_util.color_rgb_to_rgbww(
-                    *rgb_color, light.min_mireds, light.max_mireds
-                )
-            elif COLOR_MODE_HS in supported_color_modes:
+            elif (rgbw_color := params.pop(ATTR_RGBW_COLOR, None)) is not None:
+                rgb_color = color_util.color_rgbw_to_rgb(*rgbw_color)
                 params[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
-            elif COLOR_MODE_XY in supported_color_modes:
+            elif (rgbww_color := params.pop(ATTR_RGBWW_COLOR, None)) is not None:
+                rgb_color = color_util.color_rgbww_to_rgb(
+                    *rgbww_color, light.min_mireds, light.max_mireds
+                )
+                params[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
+        elif ATTR_HS_COLOR in params and ColorMode.HS not in supported_color_modes:
+            hs_color = params.pop(ATTR_HS_COLOR)
+            if ColorMode.RGB in supported_color_modes:
+                params[ATTR_RGB_COLOR] = color_util.color_hs_to_RGB(*hs_color)
+            elif ColorMode.RGBW in supported_color_modes:
+                rgb_color = color_util.color_hs_to_RGB(*hs_color)
+                params[ATTR_RGBW_COLOR] = color_util.color_rgb_to_rgbw(*rgb_color)
+            elif ColorMode.RGBWW in supported_color_modes:
+                rgb_color = color_util.color_hs_to_RGB(*hs_color)
+                params[ATTR_RGBWW_COLOR] = color_util.color_rgb_to_rgbww(
+                    *rgb_color, light.min_mireds, light.max_mireds
+                )
+            elif ColorMode.XY in supported_color_modes:
+                params[ATTR_XY_COLOR] = color_util.color_hs_to_xy(*hs_color)
+        elif ATTR_RGB_COLOR in params and ColorMode.RGB not in supported_color_modes:
+            rgb_color = params.pop(ATTR_RGB_COLOR)
+            if ColorMode.RGBW in supported_color_modes:
+                params[ATTR_RGBW_COLOR] = color_util.color_rgb_to_rgbw(*rgb_color)
+            elif ColorMode.RGBWW in supported_color_modes:
+                params[ATTR_RGBWW_COLOR] = color_util.color_rgb_to_rgbww(
+                    *rgb_color, light.min_mireds, light.max_mireds
+                )
+            elif ColorMode.HS in supported_color_modes:
+                params[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
+            elif ColorMode.XY in supported_color_modes:
                 params[ATTR_XY_COLOR] = color_util.color_RGB_to_xy(*rgb_color)
-        elif ATTR_XY_COLOR in params and COLOR_MODE_XY not in supported_color_modes:
+        elif ATTR_XY_COLOR in params and ColorMode.XY not in supported_color_modes:
             xy_color = params.pop(ATTR_XY_COLOR)
-            if COLOR_MODE_HS in supported_color_modes:
+            if ColorMode.HS in supported_color_modes:
                 params[ATTR_HS_COLOR] = color_util.color_xy_to_hs(*xy_color)
-            elif COLOR_MODE_RGB in supported_color_modes:
+            elif ColorMode.RGB in supported_color_modes:
                 params[ATTR_RGB_COLOR] = color_util.color_xy_to_RGB(*xy_color)
-            elif COLOR_MODE_RGBW in supported_color_modes:
+            elif ColorMode.RGBW in supported_color_modes:
                 rgb_color = color_util.color_xy_to_RGB(*xy_color)
                 params[ATTR_RGBW_COLOR] = color_util.color_rgb_to_rgbw(*rgb_color)
-            elif COLOR_MODE_RGBWW in supported_color_modes:
+            elif ColorMode.RGBWW in supported_color_modes:
                 rgb_color = color_util.color_xy_to_RGB(*xy_color)
                 params[ATTR_RGBWW_COLOR] = color_util.color_rgb_to_rgbww(
                     *rgb_color, light.min_mireds, light.max_mireds
                 )
+        elif ATTR_RGBW_COLOR in params and ColorMode.RGBW not in supported_color_modes:
+            rgbw_color = params.pop(ATTR_RGBW_COLOR)
+            rgb_color = color_util.color_rgbw_to_rgb(*rgbw_color)
+            if ColorMode.RGB in supported_color_modes:
+                params[ATTR_RGB_COLOR] = rgb_color
+            elif ColorMode.RGBWW in supported_color_modes:
+                params[ATTR_RGBWW_COLOR] = color_util.color_rgb_to_rgbww(
+                    *rgb_color, light.min_mireds, light.max_mireds
+                )
+            elif ColorMode.HS in supported_color_modes:
+                params[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
+            elif ColorMode.XY in supported_color_modes:
+                params[ATTR_XY_COLOR] = color_util.color_RGB_to_xy(*rgb_color)
+        elif (
+            ATTR_RGBWW_COLOR in params and ColorMode.RGBWW not in supported_color_modes
+        ):
+            rgbww_color = params.pop(ATTR_RGBWW_COLOR)
+            rgb_color = color_util.color_rgbww_to_rgb(
+                *rgbww_color, light.min_mireds, light.max_mireds
+            )
+            if ColorMode.RGB in supported_color_modes:
+                params[ATTR_RGB_COLOR] = rgb_color
+            elif ColorMode.RGBW in supported_color_modes:
+                params[ATTR_RGBW_COLOR] = color_util.color_rgb_to_rgbw(*rgb_color)
+            elif ColorMode.HS in supported_color_modes:
+                params[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
+            elif ColorMode.XY in supported_color_modes:
+                params[ATTR_XY_COLOR] = color_util.color_RGB_to_xy(*rgb_color)
+
+        # If both white and brightness are specified, override white
+        if (
+            supported_color_modes
+            and ATTR_WHITE in params
+            and ColorMode.WHITE in supported_color_modes
+        ):
+            params[ATTR_WHITE] = params.pop(ATTR_BRIGHTNESS, params[ATTR_WHITE])
 
         # Remove deprecated white value if the light supports color mode
-        if supported_color_modes:
-            params.pop(ATTR_WHITE_VALUE, None)
-
-        if params.get(ATTR_BRIGHTNESS) == 0:
+        if params.get(ATTR_BRIGHTNESS) == 0 or params.get(ATTR_WHITE) == 0:
             await async_handle_light_off_service(light, call)
         else:
-            await light.async_turn_on(**params)
+            await light.async_turn_on(**filter_turn_on_params(light, params))
 
     async def async_handle_light_off_service(light, call):
         """Handle turning off a light."""
@@ -382,7 +541,7 @@ async def async_setup(hass, config):  # noqa: C901
         if ATTR_TRANSITION not in params:
             profiles.apply_default(light.entity_id, True, params)
 
-        await light.async_turn_off(**filter_turn_off_params(params))
+        await light.async_turn_off(**filter_turn_off_params(light, params))
 
     async def async_handle_toggle_service(light, call):
         """Handle toggling a light."""
@@ -447,7 +606,7 @@ class Profile:
     transition: int | None = None
     hs_color: tuple[float, float] | None = dataclasses.field(init=False)
 
-    SCHEMA = vol.Schema(  # pylint: disable=invalid-name
+    SCHEMA = vol.Schema(
         vol.Any(
             vol.ExactSequence(
                 (
@@ -488,7 +647,7 @@ class Profile:
 class Profiles:
     """Representation of available color profiles."""
 
-    def __init__(self, hass: HomeAssistant):
+    def __init__(self, hass: HomeAssistant) -> None:
         """Initialize profiles."""
         self.hass = hass
         self.data: dict[str, Profile] = {}
@@ -504,7 +663,7 @@ class Profiles:
         for profile_path in profile_paths:
             if not os.path.isfile(profile_path):
                 continue
-            with open(profile_path) as inp:
+            with open(profile_path, encoding="utf8") as inp:
                 reader = csv.reader(inp)
 
                 # Skip the header
@@ -543,137 +702,143 @@ class Profiles:
     @callback
     def apply_profile(self, name: str, params: dict) -> None:
         """Apply a profile."""
-        profile = self.data.get(name)
-
-        if profile is None:
+        if (profile := self.data.get(name)) is None:
             return
 
-        if profile.hs_color is not None:
-            params.setdefault(ATTR_HS_COLOR, profile.hs_color)
+        color_attributes = (
+            ATTR_COLOR_NAME,
+            ATTR_COLOR_TEMP,
+            ATTR_HS_COLOR,
+            ATTR_RGB_COLOR,
+            ATTR_RGBW_COLOR,
+            ATTR_RGBWW_COLOR,
+            ATTR_XY_COLOR,
+            ATTR_WHITE,
+        )
+
+        if profile.hs_color is not None and not any(
+            color_attribute in params for color_attribute in color_attributes
+        ):
+            params[ATTR_HS_COLOR] = profile.hs_color
         if profile.brightness is not None:
             params.setdefault(ATTR_BRIGHTNESS, profile.brightness)
         if profile.transition is not None:
             params.setdefault(ATTR_TRANSITION, profile.transition)
 
 
+@dataclasses.dataclass
+class LightEntityDescription(ToggleEntityDescription):
+    """A class that describes binary sensor entities."""
+
+
 class LightEntity(ToggleEntity):
     """Base class for light entities."""
 
+    entity_description: LightEntityDescription
+    _attr_brightness: int | None = None
+    _attr_color_mode: ColorMode | str | None = None
+    _attr_color_temp: int | None = None
+    _attr_effect_list: list[str] | None = None
+    _attr_effect: str | None = None
+    _attr_hs_color: tuple[float, float] | None = None
+    _attr_max_mireds: int = 500
+    _attr_min_mireds: int = 153
+    _attr_rgb_color: tuple[int, int, int] | None = None
+    _attr_rgbw_color: tuple[int, int, int, int] | None = None
+    _attr_rgbww_color: tuple[int, int, int, int, int] | None = None
+    _attr_supported_color_modes: set[ColorMode] | set[str] | None = None
+    _attr_supported_features: int = 0
+    _attr_xy_color: tuple[float, float] | None = None
+
     @property
     def brightness(self) -> int | None:
         """Return the brightness of this light between 0..255."""
-        return None
+        return self._attr_brightness
 
     @property
-    def color_mode(self) -> str | None:
+    def color_mode(self) -> ColorMode | str | None:
         """Return the color mode of the light."""
-        return None
+        return self._attr_color_mode
 
     @property
     def _light_internal_color_mode(self) -> str:
         """Return the color mode of the light with backwards compatibility."""
-        color_mode = self.color_mode
-
-        if color_mode is None:
+        if (color_mode := self.color_mode) is None:
             # Backwards compatibility for color_mode added in 2021.4
             # Add warning in 2021.6, remove in 2021.10
             supported = self._light_internal_supported_color_modes
 
-            if (
-                COLOR_MODE_RGBW in supported
-                and self.white_value is not None
-                and self.hs_color is not None
-            ):
-                return COLOR_MODE_RGBW
-            if COLOR_MODE_HS in supported and self.hs_color is not None:
-                return COLOR_MODE_HS
-            if COLOR_MODE_COLOR_TEMP in supported and self.color_temp is not None:
-                return COLOR_MODE_COLOR_TEMP
-            if COLOR_MODE_BRIGHTNESS in supported and self.brightness is not None:
-                return COLOR_MODE_BRIGHTNESS
-            if COLOR_MODE_ONOFF in supported:
-                return COLOR_MODE_ONOFF
-            return COLOR_MODE_UNKNOWN
+            if ColorMode.HS in supported and self.hs_color is not None:
+                return ColorMode.HS
+            if ColorMode.COLOR_TEMP in supported and self.color_temp is not None:
+                return ColorMode.COLOR_TEMP
+            if ColorMode.BRIGHTNESS in supported and self.brightness is not None:
+                return ColorMode.BRIGHTNESS
+            if ColorMode.ONOFF in supported:
+                return ColorMode.ONOFF
+            return ColorMode.UNKNOWN
 
         return color_mode
 
     @property
     def hs_color(self) -> tuple[float, float] | None:
         """Return the hue and saturation color value [float, float]."""
-        return None
+        return self._attr_hs_color
 
     @property
     def xy_color(self) -> tuple[float, float] | None:
         """Return the xy color value [float, float]."""
-        return None
+        return self._attr_xy_color
 
     @property
     def rgb_color(self) -> tuple[int, int, int] | None:
         """Return the rgb color value [int, int, int]."""
-        return None
+        return self._attr_rgb_color
 
     @property
     def rgbw_color(self) -> tuple[int, int, int, int] | None:
         """Return the rgbw color value [int, int, int, int]."""
-        return None
+        return self._attr_rgbw_color
 
     @property
     def _light_internal_rgbw_color(self) -> tuple[int, int, int, int] | None:
         """Return the rgbw color value [int, int, int, int]."""
         rgbw_color = self.rgbw_color
-        if (
-            rgbw_color is None
-            and self.hs_color is not None
-            and self.white_value is not None
-        ):
-            # Backwards compatibility for rgbw_color added in 2021.4
-            # Add warning in 2021.6, remove in 2021.10
-            r, g, b = color_util.color_hs_to_RGB(  # pylint: disable=invalid-name
-                *self.hs_color
-            )
-            w = self.white_value  # pylint: disable=invalid-name
-            rgbw_color = (r, g, b, w)
-
         return rgbw_color
 
     @property
     def rgbww_color(self) -> tuple[int, int, int, int, int] | None:
         """Return the rgbww color value [int, int, int, int, int]."""
-        return None
+        return self._attr_rgbww_color
 
     @property
     def color_temp(self) -> int | None:
         """Return the CT color value in mireds."""
-        return None
+        return self._attr_color_temp
 
     @property
     def min_mireds(self) -> int:
         """Return the coldest color_temp that this light supports."""
         # Default to the Philips Hue value that HA has always assumed
         # https://developers.meethue.com/documentation/core-concepts
-        return 153
+        return self._attr_min_mireds
 
     @property
     def max_mireds(self) -> int:
         """Return the warmest color_temp that this light supports."""
         # Default to the Philips Hue value that HA has always assumed
         # https://developers.meethue.com/documentation/core-concepts
-        return 500
-
-    @property
-    def white_value(self) -> int | None:
-        """Return the white value of this light between 0..255."""
-        return None
+        return self._attr_max_mireds
 
     @property
     def effect_list(self) -> list[str] | None:
         """Return the list of supported effects."""
-        return None
+        return self._attr_effect_list
 
     @property
     def effect(self) -> str | None:
         """Return the current effect."""
-        return None
+        return self._attr_effect
 
     @property
     def capability_attributes(self):
@@ -682,42 +847,42 @@ class LightEntity(ToggleEntity):
         supported_features = self.supported_features
         supported_color_modes = self._light_internal_supported_color_modes
 
-        if COLOR_MODE_COLOR_TEMP in supported_color_modes:
+        if ColorMode.COLOR_TEMP in supported_color_modes:
             data[ATTR_MIN_MIREDS] = self.min_mireds
             data[ATTR_MAX_MIREDS] = self.max_mireds
 
-        if supported_features & SUPPORT_EFFECT:
+        if supported_features & LightEntityFeature.EFFECT:
             data[ATTR_EFFECT_LIST] = self.effect_list
 
         data[ATTR_SUPPORTED_COLOR_MODES] = sorted(supported_color_modes)
 
         return data
 
-    def _light_internal_convert_color(self, color_mode: str) -> dict:
+    def _light_internal_convert_color(self, color_mode: ColorMode | str) -> dict:
         data: dict[str, tuple] = {}
-        if color_mode == COLOR_MODE_HS and self.hs_color:
+        if color_mode == ColorMode.HS and self.hs_color:
             hs_color = self.hs_color
             data[ATTR_HS_COLOR] = (round(hs_color[0], 3), round(hs_color[1], 3))
             data[ATTR_RGB_COLOR] = color_util.color_hs_to_RGB(*hs_color)
             data[ATTR_XY_COLOR] = color_util.color_hs_to_xy(*hs_color)
-        elif color_mode == COLOR_MODE_XY and self.xy_color:
+        elif color_mode == ColorMode.XY and self.xy_color:
             xy_color = self.xy_color
             data[ATTR_HS_COLOR] = color_util.color_xy_to_hs(*xy_color)
             data[ATTR_RGB_COLOR] = color_util.color_xy_to_RGB(*xy_color)
             data[ATTR_XY_COLOR] = (round(xy_color[0], 6), round(xy_color[1], 6))
-        elif color_mode == COLOR_MODE_RGB and self.rgb_color:
+        elif color_mode == ColorMode.RGB and self.rgb_color:
             rgb_color = self.rgb_color
             data[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
             data[ATTR_RGB_COLOR] = tuple(int(x) for x in rgb_color[0:3])
             data[ATTR_XY_COLOR] = color_util.color_RGB_to_xy(*rgb_color)
-        elif color_mode == COLOR_MODE_RGBW and self._light_internal_rgbw_color:
+        elif color_mode == ColorMode.RGBW and self._light_internal_rgbw_color:
             rgbw_color = self._light_internal_rgbw_color
             rgb_color = color_util.color_rgbw_to_rgb(*rgbw_color)
             data[ATTR_HS_COLOR] = color_util.color_RGB_to_hs(*rgb_color)
             data[ATTR_RGB_COLOR] = tuple(int(x) for x in rgb_color[0:3])
             data[ATTR_RGBW_COLOR] = tuple(int(x) for x in rgbw_color[0:4])
             data[ATTR_XY_COLOR] = color_util.color_RGB_to_xy(*rgb_color)
-        elif color_mode == COLOR_MODE_RGBWW and self.rgbww_color:
+        elif color_mode == ColorMode.RGBWW and self.rgbww_color:
             rgbww_color = self.rgbww_color
             rgb_color = color_util.color_rgbww_to_rgb(
                 *rgbww_color, self.min_mireds, self.max_mireds
@@ -726,6 +891,13 @@ class LightEntity(ToggleEntity):
             data[ATTR_RGB_COLOR] = tuple(int(x) for x in rgb_color[0:3])
             data[ATTR_RGBWW_COLOR] = tuple(int(x) for x in rgbww_color[0:5])
             data[ATTR_XY_COLOR] = color_util.color_RGB_to_xy(*rgb_color)
+        elif color_mode == ColorMode.COLOR_TEMP and self.color_temp:
+            hs_color = color_util.color_temperature_to_hs(
+                color_util.color_temperature_mired_to_kelvin(self.color_temp)
+            )
+            data[ATTR_HS_COLOR] = (round(hs_color[0], 3), round(hs_color[1], 3))
+            data[ATTR_RGB_COLOR] = color_util.color_hs_to_RGB(*hs_color)
+            data[ATTR_XY_COLOR] = color_util.color_hs_to_xy(*hs_color)
         return data
 
     @final
@@ -757,10 +929,10 @@ class LightEntity(ToggleEntity):
             # Add warning in 2021.6, remove in 2021.10
             data[ATTR_BRIGHTNESS] = self.brightness
 
-        if color_mode == COLOR_MODE_COLOR_TEMP:
+        if color_mode == ColorMode.COLOR_TEMP:
             data[ATTR_COLOR_TEMP] = self.color_temp
 
-        if color_mode in COLOR_MODES_COLOR:
+        if color_mode in COLOR_MODES_COLOR or color_mode == ColorMode.COLOR_TEMP:
             data.update(self._light_internal_convert_color(color_mode))
 
         if supported_features & SUPPORT_COLOR_TEMP and not self.supported_color_modes:
@@ -768,64 +940,43 @@ class LightEntity(ToggleEntity):
             # Add warning in 2021.6, remove in 2021.10
             data[ATTR_COLOR_TEMP] = self.color_temp
 
-        if supported_features & SUPPORT_WHITE_VALUE and not self.supported_color_modes:
-            # Backwards compatibility
-            # Add warning in 2021.6, remove in 2021.10
-            data[ATTR_WHITE_VALUE] = self.white_value
-            if self.hs_color is not None:
-                data.update(self._light_internal_convert_color(COLOR_MODE_HS))
-
-        if supported_features & SUPPORT_EFFECT:
+        if supported_features & LightEntityFeature.EFFECT:
             data[ATTR_EFFECT] = self.effect
 
         return {key: val for key, val in data.items() if val is not None}
 
     @property
-    def _light_internal_supported_color_modes(self) -> set:
+    def _light_internal_supported_color_modes(self) -> set[ColorMode] | set[str]:
         """Calculate supported color modes with backwards compatibility."""
-        supported_color_modes = self.supported_color_modes
+        if self.supported_color_modes is not None:
+            return self.supported_color_modes
 
-        if supported_color_modes is None:
-            # Backwards compatibility for supported_color_modes added in 2021.4
-            # Add warning in 2021.6, remove in 2021.10
-            supported_features = self.supported_features
-            supported_color_modes = set()
+        # Backwards compatibility for supported_color_modes added in 2021.4
+        # Add warning in 2021.6, remove in 2021.10
+        supported_features = self.supported_features
+        supported_color_modes: set[ColorMode] = set()
 
-            if supported_features & SUPPORT_COLOR_TEMP:
-                supported_color_modes.add(COLOR_MODE_COLOR_TEMP)
-            if supported_features & SUPPORT_COLOR:
-                supported_color_modes.add(COLOR_MODE_HS)
-            if supported_features & SUPPORT_WHITE_VALUE:
-                supported_color_modes.add(COLOR_MODE_RGBW)
-            if supported_features & SUPPORT_BRIGHTNESS and not supported_color_modes:
-                supported_color_modes = {COLOR_MODE_BRIGHTNESS}
+        if supported_features & SUPPORT_COLOR_TEMP:
+            supported_color_modes.add(ColorMode.COLOR_TEMP)
+        if supported_features & SUPPORT_COLOR:
+            supported_color_modes.add(ColorMode.HS)
+        if supported_features & SUPPORT_BRIGHTNESS and not supported_color_modes:
+            supported_color_modes = {ColorMode.BRIGHTNESS}
 
-            if not supported_color_modes:
-                supported_color_modes = {COLOR_MODE_ONOFF}
+        if not supported_color_modes:
+            supported_color_modes = {ColorMode.ONOFF}
 
         return supported_color_modes
 
     @property
-    def supported_color_modes(self) -> set | None:
+    def supported_color_modes(self) -> set[ColorMode] | set[str] | None:
         """Flag supported color modes."""
-        return None
+        return self._attr_supported_color_modes
 
     @property
     def supported_features(self) -> int:
         """Flag supported features."""
-        return 0
-
-
-class Light(LightEntity):
-    """Representation of a light (for backwards compatibility)."""
-
-    def __init_subclass__(cls, **kwargs):
-        """Print deprecation warning."""
-        super().__init_subclass__(**kwargs)
-        _LOGGER.warning(
-            "Light is deprecated, modify %s to extend LightEntity",
-            cls.__name__,
-        )
+        return self._attr_supported_features
 
 
 def legacy_supported_features(
@@ -839,7 +990,7 @@ def legacy_supported_features(
         supported_features |= SUPPORT_COLOR
     if any(mode in supported_color_modes for mode in COLOR_MODES_BRIGHTNESS):
         supported_features |= SUPPORT_BRIGHTNESS
-    if COLOR_MODE_COLOR_TEMP in supported_color_modes:
+    if ColorMode.COLOR_TEMP in supported_color_modes:
         supported_features |= SUPPORT_COLOR_TEMP
 
     return supported_features
