@@ -80,6 +80,8 @@ async def async_setup_platform(
 class MoldIndicator(SensorEntity):
     """Represents a MoldIndication sensor."""
 
+    _attr_should_poll = False
+
     def __init__(
         self,
         name,
@@ -205,9 +207,8 @@ class MoldIndicator(SensorEntity):
             return None
 
         unit = state.attributes.get(ATTR_UNIT_OF_MEASUREMENT)
-        temp = util.convert(state.state, float)
 
-        if temp is None:
+        if (temp := util.convert(state.state, float)) is None:
             _LOGGER.error(
                 "Unable to parse temperature sensor %s with state: %s",
                 state.entity_id,
@@ -354,11 +355,6 @@ class MoldIndicator(SensorEntity):
 
         _LOGGER.debug("Mold indicator humidity: %s", self._state)
 
-    @property
-    def should_poll(self):
-        """Return the polling state."""
-        return False
-
     @property
     def name(self):
         """Return the name."""
