@@ -4,19 +4,21 @@ from __future__ import annotations
 import asyncio
 from collections import deque
 import functools
-import json
 import logging
 import re
 import time
+from typing import TYPE_CHECKING
 
 from homeassistant.const import CONF_DEVICE, CONF_PLATFORM
 from homeassistant.core import HomeAssistant
-from homeassistant.data_entry_flow import RESULT_TYPE_ABORT
+from homeassistant.data_entry_flow import FlowResultType
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
 )
+from homeassistant.helpers.json import json_loads
+from homeassistant.helpers.service_info.mqtt import MqttServiceInfo
 from homeassistant.loader import async_get_mqtt
 
 from .. import mqtt
@@ -27,11 +29,13 @@ from .const import (
     ATTR_DISCOVERY_TOPIC,
     CONF_AVAILABILITY,
     CONF_TOPIC,
-    CONFIG_ENTRY_IS_SETUP,
-    DATA_CONFIG_ENTRY_LOCK,
+    DATA_MQTT,
     DOMAIN,
 )
 
+if TYPE_CHECKING:
+    from .mixins import MqttData
+
 _LOGGER = logging.getLogger(__name__)
 
 TOPIC_MATCHER = re.compile(
@@ -70,7 +74,6 @@ INTEGRATION_UNSUBSCRIBE = "mqtt_integration_discovery_unsubscribe"
 MQTT_DISCOVERY_UPDATED = "mqtt_discovery_updated_{}"
 MQTT_DISCOVERY_NEW = "mqtt_discovery_new_{}_{}"
 MQTT_DISCOVERY_DONE = "mqtt_discovery_done_{}"
-LAST_DISCOVERY = "mqtt_last_discovery"
 
 TOPIC_BASE = "~"
 
@@ -81,12 +84,12 @@ class MQTTConfig(dict):
     discovery_data: dict
 
 
-def clear_discovery_hash(hass: HomeAssistant, discovery_hash: tuple) -> None:
+def clear_discovery_hash(hass: HomeAssistant, discovery_hash: tuple[str, str]) -> None:
     """Clear entry in ALREADY_DISCOVERED list."""
     del hass.data[ALREADY_DISCOVERED][discovery_hash]
 
 
-def set_discovery_hash(hass: HomeAssistant, discovery_hash: tuple):
+def set_discovery_hash(hass: HomeAssistant, discovery_hash: tuple[str, str]):
     """Clear entry in ALREADY_DISCOVERED list."""
     hass.data[ALREADY_DISCOVERED][discovery_hash] = {}
 
@@ -95,11 +98,12 @@ async def async_start(  # noqa: C901
     hass: HomeAssistant, discovery_topic, config_entry=None
 ) -> None:
     """Start MQTT Discovery."""
+    mqtt_data: MqttData = hass.data[DATA_MQTT]
     mqtt_integrations = {}
 
     async def async_discovery_message_received(msg):
         """Process the received message."""
-        hass.data[LAST_DISCOVERY] = time.time()
+        mqtt_data.last_discovery = time.time()
         payload = msg.payload
         topic = msg.topic
         topic_trimmed = topic.replace(f"{discovery_topic}/", "", 1)
@@ -107,7 +111,10 @@ async def async_start(  # noqa: C901
         if not (match := TOPIC_MATCHER.match(topic_trimmed)):
             if topic_trimmed.endswith("config"):
                 _LOGGER.warning(
-                    "Received message on illegal discovery topic '%s'", topic
+                    "Received message on illegal discovery topic '%s'. The topic contains "
+                    "not allowed characters. For more information see "
+                    "https://www.home-assistant.io/docs/mqtt/discovery/#discovery-topic",
+                    topic,
                 )
             return
 
@@ -119,7 +126,7 @@ async def async_start(  # noqa: C901
 
         if payload:
             try:
-                payload = json.loads(payload)
+                payload = json_loads(payload)
             except ValueError:
                 _LOGGER.warning("Unable to parse JSON %s: '%s'", object_id, payload)
                 return
@@ -227,28 +234,6 @@ async def async_start(  # noqa: C901
             # Add component
             _LOGGER.info("Found new component: %s %s", component, discovery_id)
             hass.data[ALREADY_DISCOVERED][discovery_hash] = None
-
-            config_entries_key = f"{component}.mqtt"
-            async with hass.data[DATA_CONFIG_ENTRY_LOCK]:
-                if config_entries_key not in hass.data[CONFIG_ENTRY_IS_SETUP]:
-                    if component == "device_automation":
-                        # Local import to avoid circular dependencies
-                        # pylint: disable-next=import-outside-toplevel
-                        from . import device_automation
-
-                        await device_automation.async_setup_entry(hass, config_entry)
-                    elif component == "tag":
-                        # Local import to avoid circular dependencies
-                        # pylint: disable-next=import-outside-toplevel
-                        from . import tag
-
-                        await tag.async_setup_entry(hass, config_entry)
-                    else:
-                        await hass.config_entries.async_forward_entry_setup(
-                            config_entry, component
-                        )
-                    hass.data[CONFIG_ENTRY_IS_SETUP].add(config_entries_key)
-
             async_dispatcher_send(
                 hass, MQTT_DISCOVERY_NEW.format(component, "mqtt"), payload
             )
@@ -258,8 +243,7 @@ async def async_start(  # noqa: C901
                 hass, MQTT_DISCOVERY_DONE.format(discovery_hash), None
             )
 
-    hass.data[DATA_CONFIG_FLOW_LOCK] = asyncio.Lock()
-
+    hass.data.setdefault(DATA_CONFIG_FLOW_LOCK, asyncio.Lock())
     hass.data[ALREADY_DISCOVERED] = {}
     hass.data[PENDING_DISCOVERED] = {}
 
@@ -274,7 +258,7 @@ async def async_start(  # noqa: C901
         )
     )
 
-    hass.data[LAST_DISCOVERY] = time.time()
+    mqtt_data.last_discovery = time.time()
     mqtt_integrations = await async_get_mqtt(hass)
 
     hass.data[INTEGRATION_UNSUBSCRIBE] = {}
@@ -292,7 +276,7 @@ async def async_start(  # noqa: C901
                 if key not in hass.data[INTEGRATION_UNSUBSCRIBE]:
                     return
 
-                data = mqtt.MqttServiceInfo(
+                data = MqttServiceInfo(
                     topic=msg.topic,
                     payload=msg.payload,
                     qos=msg.qos,
@@ -305,7 +289,7 @@ async def async_start(  # noqa: C901
                 )
                 if (
                     result
-                    and result["type"] == RESULT_TYPE_ABORT
+                    and result["type"] == FlowResultType.ABORT
                     and result["reason"]
                     in ("already_configured", "single_instance_allowed")
                 ):
