@@ -3,16 +3,18 @@ from __future__ import annotations
 
 from abc import abstractmethod
 import asyncio
-from collections.abc import Callable
-import json
+from collections.abc import Callable, Coroutine
+from dataclasses import dataclass, field
+from functools import partial
 import logging
-from typing import Any, Protocol, cast, final
+from typing import TYPE_CHECKING, Any, Protocol, cast, final
 
 import voluptuous as vol
 
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_CONFIGURATION_URL,
+    ATTR_HW_VERSION,
     ATTR_MANUFACTURER,
     ATTR_MODEL,
     ATTR_NAME,
@@ -27,11 +29,16 @@ from homeassistant.const import (
     CONF_UNIQUE_ID,
     CONF_VALUE_TEMPLATE,
 )
-from homeassistant.core import CALLBACK_TYPE, Event, HomeAssistant, callback
+from homeassistant.core import (
+    CALLBACK_TYPE,
+    Event,
+    HomeAssistant,
+    async_get_hass,
+    callback,
+)
 from homeassistant.helpers import (
     config_validation as cv,
     device_registry as dr,
-    discovery,
     entity_registry as er,
 )
 from homeassistant.helpers.device_registry import EVENT_DEVICE_REGISTRY_UPDATED
@@ -47,10 +54,13 @@ from homeassistant.helpers.entity import (
     async_generate_entity_id,
 )
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.event import async_track_entity_registry_updated_event
+from homeassistant.helpers.issue_registry import IssueSeverity, async_create_issue
+from homeassistant.helpers.json import json_loads
 from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 from . import debug_info, subscription
-from .client import async_publish
+from .client import MQTT, Subscription, async_publish
 from .const import (
     ATTR_DISCOVERY_HASH,
     ATTR_DISCOVERY_PAYLOAD,
@@ -60,16 +70,12 @@ from .const import (
     CONF_QOS,
     CONF_TOPIC,
     DATA_MQTT,
-    DATA_MQTT_CONFIG,
-    DATA_MQTT_RELOAD_NEEDED,
-    DATA_MQTT_UPDATED_CONFIG,
     DEFAULT_ENCODING,
     DEFAULT_PAYLOAD_AVAILABLE,
     DEFAULT_PAYLOAD_NOT_AVAILABLE,
     DOMAIN,
     MQTT_CONNECTED,
     MQTT_DISCONNECTED,
-    MQTT_RELOADED,
 )
 from .debug_info import log_message, log_messages
 from .discovery import (
@@ -85,7 +91,10 @@ from .subscription import (
     async_subscribe_topics,
     async_unsubscribe_topics,
 )
-from .util import valid_subscribe_topic
+from .util import mqtt_config_entry_enabled, valid_subscribe_topic
+
+if TYPE_CHECKING:
+    from .device_trigger import Trigger
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -107,6 +116,7 @@ CONF_JSON_ATTRS_TEMPLATE = "json_attributes_template"
 CONF_IDENTIFIERS = "identifiers"
 CONF_CONNECTIONS = "connections"
 CONF_MANUFACTURER = "manufacturer"
+CONF_HW_VERSION = "hw_version"
 CONF_SW_VERSION = "sw_version"
 CONF_VIA_DEVICE = "via_device"
 CONF_DEPRECATED_VIA_HUB = "via_hub"
@@ -199,6 +209,7 @@ MQTT_ENTITY_DEVICE_INFO_SCHEMA = vol.All(
             vol.Optional(CONF_MANUFACTURER): cv.string,
             vol.Optional(CONF_MODEL): cv.string,
             vol.Optional(CONF_NAME): cv.string,
+            vol.Optional(CONF_HW_VERSION): cv.string,
             vol.Optional(CONF_SW_VERSION): cv.string,
             vol.Optional(CONF_VIA_DEVICE): cv.string,
             vol.Optional(CONF_SUGGESTED_AREA): cv.string,
@@ -242,11 +253,46 @@ def warn_for_legacy_schema(domain: str) -> Callable:
             domain,
         )
         warned.add(domain)
+        # Register a repair
+        async_create_issue(
+            async_get_hass(),
+            DOMAIN,
+            f"deprecated_yaml_{domain}",
+            breaks_in_ha_version="2022.12.0",  # Warning first added in 2022.6.0
+            is_fixable=False,
+            severity=IssueSeverity.WARNING,
+            translation_key="deprecated_yaml",
+            translation_placeholders={
+                "more_info_url": f"https://www.home-assistant.io/integrations/{domain}.mqtt/#new_format",
+                "platform": domain,
+            },
+        )
         return config
 
     return validator
 
 
+@dataclass
+class MqttData:
+    """Keep the MQTT entry data."""
+
+    client: MQTT | None = None
+    config: ConfigType | None = None
+    device_triggers: dict[str, Trigger] = field(default_factory=dict)
+    discovery_registry_hooks: dict[tuple[str, str], CALLBACK_TYPE] = field(
+        default_factory=dict
+    )
+    last_discovery: float = 0.0
+    reload_dispatchers: list[CALLBACK_TYPE] = field(default_factory=list)
+    reload_entry: bool = False
+    reload_handlers: dict[str, Callable[[], Coroutine[Any, Any, None]]] = field(
+        default_factory=dict
+    )
+    reload_needed: bool = False
+    subscriptions_to_restore: list[Subscription] = field(default_factory=list)
+    updated_config: ConfigType = field(default_factory=dict)
+
+
 class SetupEntity(Protocol):
     """Protocol type for async_setup_entities."""
 
@@ -261,45 +307,16 @@ class SetupEntity(Protocol):
         """Define setup_entities type."""
 
 
-async def async_setup_platform_discovery(
-    hass: HomeAssistant, platform_domain: str
-) -> CALLBACK_TYPE:
-    """Set up platform discovery for manual config."""
-
-    async def _async_discover_entities() -> None:
-        """Discover entities for a platform."""
-        if DATA_MQTT_UPDATED_CONFIG in hass.data:
-            # The platform has been reloaded
-            config_yaml = hass.data[DATA_MQTT_UPDATED_CONFIG]
-        else:
-            config_yaml = hass.data.get(DATA_MQTT_CONFIG, {})
-        if not config_yaml:
-            return
-        if platform_domain not in config_yaml:
-            return
-        await asyncio.gather(
-            *(
-                discovery.async_load_platform(hass, platform_domain, DOMAIN, config, {})
-                for config in await async_get_platform_config_from_yaml(
-                    hass, platform_domain, config_yaml
-                )
-            )
-        )
-
-    unsub = async_dispatcher_connect(hass, MQTT_RELOADED, _async_discover_entities)
-    await _async_discover_entities()
-    return unsub
-
-
 async def async_get_platform_config_from_yaml(
     hass: HomeAssistant,
     platform_domain: str,
-    config_yaml: ConfigType = None,
+    config_yaml: ConfigType | None = None,
 ) -> list[ConfigType]:
     """Return a list of validated configurations for the domain."""
 
+    mqtt_data: MqttData = hass.data[DATA_MQTT]
     if config_yaml is None:
-        config_yaml = hass.data.get(DATA_MQTT_CONFIG)
+        config_yaml = mqtt_data.config
     if not config_yaml:
         return []
     if not (platform_configs := config_yaml.get(platform_domain)):
@@ -307,14 +324,28 @@ async def async_get_platform_config_from_yaml(
     return platform_configs
 
 
-async def async_setup_entry_helper(hass, domain, async_setup, schema):
+async def async_setup_entry_helper(
+    hass: HomeAssistant,
+    domain: str,
+    async_setup: partial[Coroutine[HomeAssistant, str, None]],
+    discovery_schema: vol.Schema,
+) -> None:
     """Set up entity, automation or tag creation dynamically through MQTT discovery."""
+    mqtt_data: MqttData = hass.data[DATA_MQTT]
 
     async def async_discover(discovery_payload):
         """Discover and add an MQTT entity, automation or tag."""
+        if not mqtt_config_entry_enabled(hass):
+            _LOGGER.warning(
+                "MQTT integration is disabled, skipping setup of discovered item "
+                "MQTT %s, payload %s",
+                domain,
+                discovery_payload,
+            )
+            return
         discovery_data = discovery_payload.discovery_data
         try:
-            config = schema(discovery_payload)
+            config = discovery_schema(discovery_payload)
             await async_setup(config, discovery_data=discovery_data)
         except Exception:
             discovery_hash = discovery_data[ATTR_DISCOVERY_HASH]
@@ -324,10 +355,37 @@ async def async_setup_entry_helper(hass, domain, async_setup, schema):
             )
             raise
 
-    async_dispatcher_connect(
-        hass, MQTT_DISCOVERY_NEW.format(domain, "mqtt"), async_discover
+    mqtt_data.reload_dispatchers.append(
+        async_dispatcher_connect(
+            hass, MQTT_DISCOVERY_NEW.format(domain, "mqtt"), async_discover
+        )
     )
 
+    async def _async_setup_entities() -> None:
+        """Set up MQTT items from configuration.yaml."""
+        mqtt_data: MqttData = hass.data[DATA_MQTT]
+        if mqtt_data.updated_config:
+            # The platform has been reloaded
+            config_yaml = mqtt_data.updated_config
+        else:
+            config_yaml = mqtt_data.config or {}
+        if not config_yaml:
+            return
+        if domain not in config_yaml:
+            return
+        await asyncio.gather(
+            *[
+                async_setup(config)
+                for config in await async_get_platform_config_from_yaml(
+                    hass, domain, config_yaml
+                )
+            ]
+        )
+
+    # discover manual configured MQTT items
+    mqtt_data.reload_handlers[domain] = _async_setup_entities
+    await _async_setup_entities()
+
 
 async def async_setup_platform_helper(
     hass: HomeAssistant,
@@ -336,16 +394,24 @@ async def async_setup_platform_helper(
     async_add_entities: AddEntitiesCallback,
     async_setup_entities: SetupEntity,
 ) -> None:
-    """Return true if platform setup should be aborted."""
-    if not bool(hass.config_entries.async_entries(DOMAIN)):
-        hass.data[DATA_MQTT_RELOAD_NEEDED] = None
-        _LOGGER.warning(
-            "MQTT integration is not setup, skipping setup of manually configured "
-            "MQTT %s",
+    """Help to set up the platform for manual configured MQTT entities."""
+    mqtt_data: MqttData = hass.data[DATA_MQTT]
+    if mqtt_data.reload_entry:
+        _LOGGER.debug(
+            "MQTT integration is %s, skipping setup of manually configured MQTT items while unloading the config entry",
             platform_domain,
         )
         return
-    await async_setup_entities(hass, async_add_entities, config)
+    if not (entry_status := mqtt_config_entry_enabled(hass)):
+        _LOGGER.warning(
+            "MQTT integration is %s, skipping setup of manually configured MQTT %s",
+            "not setup" if entry_status is None else "disabled",
+            platform_domain,
+        )
+        return
+    # Ensure we set config_entry when entries are set up to enable clean up
+    config_entry: ConfigEntry = hass.config_entries.async_entries(DOMAIN)[0]
+    await async_setup_entities(hass, async_add_entities, config, config_entry)
 
 
 def init_entity_id_from_config(hass, entity, config, entity_id_format):
@@ -363,7 +429,7 @@ class MqttAttributes(Entity):
 
     def __init__(self, config: dict) -> None:
         """Initialize the JSON attributes mixin."""
-        self._attributes: dict | None = None
+        self._attributes: dict[str, Any] | None = None
         self._attributes_sub_state = None
         self._attributes_config = config
 
@@ -393,7 +459,7 @@ class MqttAttributes(Entity):
         def attributes_message_received(msg: ReceiveMessage) -> None:
             try:
                 payload = attr_tpl(msg.payload)
-                json_dict = json.loads(payload) if isinstance(payload, str) else None
+                json_dict = json_loads(payload) if isinstance(payload, str) else None
                 if isinstance(json_dict, dict):
                     filtered_dict = {
                         k: v
@@ -434,7 +500,7 @@ class MqttAttributes(Entity):
         )
 
     @property
-    def extra_state_attributes(self):
+    def extra_state_attributes(self) -> dict[str, Any] | None:
         """Return the state attributes."""
         return self._attributes
 
@@ -555,7 +621,10 @@ class MqttAvailability(Entity):
     @property
     def available(self) -> bool:
         """Return if the device is available."""
-        if not self.hass.data[DATA_MQTT].connected and not self.hass.is_stopping:
+        mqtt_data: MqttData = self.hass.data[DATA_MQTT]
+        assert mqtt_data.client is not None
+        client = mqtt_data.client
+        if not client.connected and not self.hass.is_stopping:
             return False
         if not self._avail_topics:
             return True
@@ -590,7 +659,7 @@ async def cleanup_device_registry(
         )
 
 
-def get_discovery_hash(discovery_data: dict) -> tuple:
+def get_discovery_hash(discovery_data: dict) -> tuple[str, str]:
     """Get the discovery hash from the discovery data."""
     return discovery_data[ATTR_DISCOVERY_HASH]
 
@@ -620,6 +689,17 @@ async def async_remove_discovery_payload(hass: HomeAssistant, discovery_data: di
     await async_publish(hass, discovery_topic, "", retain=True)
 
 
+async def async_clear_discovery_topic_if_entity_removed(
+    hass: HomeAssistant,
+    discovery_data: dict[str, Any],
+    event: Event,
+) -> None:
+    """Clear the discovery topic if the entity is removed."""
+    if event.data["action"] == "remove":
+        # publish empty payload to config topic to avoid re-adding
+        await async_remove_discovery_payload(hass, discovery_data)
+
+
 class MqttDiscoveryDeviceUpdate:
     """Add support for auto discovery for platforms without an entity."""
 
@@ -648,6 +728,7 @@ class MqttDiscoveryDeviceUpdate:
             MQTT_DISCOVERY_UPDATED.format(discovery_hash),
             self.async_discovery_update,
         )
+        config_entry.async_on_unload(self._entry_unload)
         if device_id is not None:
             self._remove_device_updated = hass.bus.async_listen(
                 EVENT_DEVICE_REGISTRY_UPDATED, self._async_device_removed
@@ -658,6 +739,14 @@ class MqttDiscoveryDeviceUpdate:
             discovery_hash,
         )
 
+    @callback
+    def _entry_unload(self, *_: Any) -> None:
+        """Handle cleanup when the config entry is unloaded."""
+        stop_discovery_updates(
+            self.hass, self._discovery_data, self._remove_discovery_updated
+        )
+        self._config_entry.async_create_task(self.hass, self.async_tear_down())
+
     async def async_discovery_update(
         self,
         discovery_payload: DiscoveryInfoType | None,
@@ -742,12 +831,24 @@ class MqttDiscoveryDeviceUpdate:
 class MqttDiscoveryUpdate(Entity):
     """Mixin used to handle updated discovery message for entity based platforms."""
 
-    def __init__(self, discovery_data, discovery_update=None) -> None:
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        discovery_data: dict | None,
+        discovery_update: Callable | None = None,
+    ) -> None:
         """Initialize the discovery update mixin."""
         self._discovery_data = discovery_data
         self._discovery_update = discovery_update
         self._remove_discovery_updated: Callable | None = None
         self._removed_from_hass = False
+        if discovery_data is None:
+            return
+        mqtt_data: MqttData = hass.data[DATA_MQTT]
+        self._registry_hooks = mqtt_data.discovery_registry_hooks
+        discovery_hash: tuple[str, str] = discovery_data[ATTR_DISCOVERY_HASH]
+        if discovery_hash in self._registry_hooks:
+            self._registry_hooks.pop(discovery_hash)()
 
     async def async_added_to_hass(self) -> None:
         """Subscribe to discovery updates."""
@@ -810,7 +911,7 @@ class MqttDiscoveryUpdate(Entity):
 
     async def async_removed_from_registry(self) -> None:
         """Clear retained discovery topic in broker."""
-        if not self._removed_from_hass:
+        if not self._removed_from_hass and self._discovery_data is not None:
             # Stop subscribing to discovery updates to not trigger when we clear the
             # discovery topic
             self._cleanup_discovery_on_remove()
@@ -821,7 +922,20 @@ class MqttDiscoveryUpdate(Entity):
     @callback
     def add_to_platform_abort(self) -> None:
         """Abort adding an entity to a platform."""
-        if self._discovery_data:
+        if self._discovery_data is not None:
+            discovery_hash: tuple[str, str] = self._discovery_data[ATTR_DISCOVERY_HASH]
+            if self.registry_entry is not None:
+                self._registry_hooks[
+                    discovery_hash
+                ] = async_track_entity_registry_updated_event(
+                    self.hass,
+                    self.entity_id,
+                    partial(
+                        async_clear_discovery_topic_if_entity_removed,
+                        self.hass,
+                        self._discovery_data,
+                    ),
+                )
             stop_discovery_updates(self.hass, self._discovery_data)
             send_discovery_done(self.hass, self._discovery_data)
         super().add_to_platform_abort()
@@ -858,6 +972,9 @@ def device_info_from_config(config) -> DeviceInfo | None:
     if CONF_NAME in config:
         info[ATTR_NAME] = config[CONF_NAME]
 
+    if CONF_HW_VERSION in config:
+        info[ATTR_HW_VERSION] = config[CONF_HW_VERSION]
+
     if CONF_SW_VERSION in config:
         info[ATTR_SW_VERSION] = config[CONF_SW_VERSION]
 
@@ -907,6 +1024,7 @@ class MqttEntity(
 ):
     """Representation of an MQTT entity."""
 
+    _attr_should_poll = False
     _entity_id_format: str
 
     def __init__(self, hass, config, config_entry, discovery_data):
@@ -925,7 +1043,7 @@ class MqttEntity(
         # Initialize mixin classes
         MqttAttributes.__init__(self, config)
         MqttAvailability.__init__(self, config)
-        MqttDiscoveryUpdate.__init__(self, discovery_data, self.discovery_update)
+        MqttDiscoveryUpdate.__init__(self, hass, discovery_data, self.discovery_update)
         MqttEntityDeviceInfo.__init__(self, config.get(CONF_DEVICE), config_entry)
 
     def _init_entity_id(self):
@@ -1033,11 +1151,6 @@ class MqttEntity(
         """Return the name of the device if any."""
         return self._config.get(CONF_NAME)
 
-    @property
-    def should_poll(self):
-        """No polling needed."""
-        return False
-
     @property
     def unique_id(self):
         """Return a unique ID."""
