@@ -22,6 +22,10 @@ from google_nest_sdm.exceptions import (
 import voluptuous as vol
 
 from homeassistant.auth.permissions.const import POLICY_READ
+from homeassistant.components.application_credentials import (
+    ClientCredential,
+    async_import_client_credential,
+)
 from homeassistant.components.camera import Image, img_util
 from homeassistant.components.http.const import KEY_HASS_USER
 from homeassistant.components.http.view import HomeAssistantView
@@ -42,32 +46,42 @@ from homeassistant.exceptions import (
     HomeAssistantError,
     Unauthorized,
 )
-from homeassistant.helpers import config_validation as cv
+from homeassistant.helpers import (
+    config_validation as cv,
+    device_registry as dr,
+    entity_registry as er,
+)
 from homeassistant.helpers.entity_registry import async_entries_for_device
+from homeassistant.helpers.issue_registry import (
+    IssueSeverity,
+    async_create_issue,
+    async_delete_issue,
+)
 from homeassistant.helpers.typing import ConfigType
 
 from . import api, config_flow
 from .const import (
     CONF_PROJECT_ID,
     CONF_SUBSCRIBER_ID,
+    CONF_SUBSCRIBER_ID_IMPORTED,
+    DATA_DEVICE_MANAGER,
     DATA_NEST_CONFIG,
     DATA_SDM,
     DATA_SUBSCRIBER,
     DOMAIN,
+    INSTALLED_AUTH_DOMAIN,
+    WEB_AUTH_DOMAIN,
 )
 from .events import EVENT_NAME_MAP, NEST_EVENT
 from .legacy import async_setup_legacy, async_setup_legacy_entry
 from .media_source import (
     async_get_media_event_store,
+    async_get_media_source_devices,
     async_get_transcoder,
-    get_media_source_devices,
 )
 
 _LOGGER = logging.getLogger(__name__)
 
-DATA_NEST_UNAVAILABLE = "nest_unavailable"
-
-NEST_SETUP_NOTIFICATION = "nest_setup"
 
 SENSOR_SCHEMA = vol.Schema(
     {vol.Optional(CONF_MONITORED_CONDITIONS): vol.All(cv.ensure_list)}
@@ -107,20 +121,22 @@ THUMBNAIL_SIZE_PX = 175
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up Nest components with dispatch between old/new flows."""
     hass.data[DOMAIN] = {}
-    hass.data[DOMAIN][DATA_NEST_CONFIG] = config.get(DOMAIN)
+
+    hass.http.register_view(NestEventMediaView(hass))
+    hass.http.register_view(NestEventMediaThumbnailView(hass))
 
     if DOMAIN not in config:
-        return True
+        return True  # ConfigMode.SDM_APPLICATION_CREDENTIALS
+
+    # Note that configuration.yaml deprecation warnings are handled in the
+    # config entry since we don't know what type of credentials we have and
+    # whether or not they can be imported.
+    hass.data[DOMAIN][DATA_NEST_CONFIG] = config[DOMAIN]
 
     config_mode = config_flow.get_config_mode(hass)
     if config_mode == config_flow.ConfigMode.LEGACY:
         return await async_setup_legacy(hass, config)
 
-    config_flow.register_flow_implementation_from_config(hass, config)
-
-    hass.http.register_view(NestEventMediaView(hass))
-    hass.http.register_view(NestEventMediaThumbnailView(hass))
-
     return True
 
 
@@ -145,7 +161,7 @@ class SignalUpdateCallback:
         if not (events := event_message.resource_update_events):
             return
         _LOGGER.debug("Event Update %s", events.keys())
-        device_registry = await self._hass.helpers.device_registry.async_get_registry()
+        device_registry = dr.async_get(self._hass)
         device_entry = device_registry.async_get_device({(DOMAIN, device_id)})
         if not device_entry:
             return
@@ -165,10 +181,19 @@ class SignalUpdateCallback:
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Nest from a config entry with dispatch between old/new flows."""
-
-    if DATA_SDM not in entry.data:
+    config_mode = config_flow.get_config_mode(hass)
+    if DATA_SDM not in entry.data or config_mode == config_flow.ConfigMode.LEGACY:
         return await async_setup_legacy_entry(hass, entry)
 
+    if config_mode == config_flow.ConfigMode.SDM:
+        await async_import_config(hass, entry)
+    elif entry.unique_id != entry.data[CONF_PROJECT_ID]:
+        hass.config_entries.async_update_entry(
+            entry, unique_id=entry.data[CONF_PROJECT_ID]
+        )
+
+    async_delete_issue(hass, DOMAIN, "removed_app_auth")
+
     subscriber = await api.new_subscriber(hass, entry)
     if not subscriber:
         return False
@@ -187,55 +212,117 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     try:
         await subscriber.start_async()
     except AuthException as err:
-        _LOGGER.debug("Subscriber authentication error: %s", err)
-        raise ConfigEntryAuthFailed from err
+        raise ConfigEntryAuthFailed(
+            f"Subscriber authentication error: {str(err)}"
+        ) from err
     except ConfigurationException as err:
         _LOGGER.error("Configuration error: %s", err)
         subscriber.stop_async()
         return False
     except SubscriberException as err:
-        if DATA_NEST_UNAVAILABLE not in hass.data[DOMAIN]:
-            _LOGGER.error("Subscriber error: %s", err)
-            hass.data[DOMAIN][DATA_NEST_UNAVAILABLE] = True
         subscriber.stop_async()
-        raise ConfigEntryNotReady from err
+        raise ConfigEntryNotReady(f"Subscriber error: {str(err)}") from err
 
     try:
-        await subscriber.async_get_device_manager()
+        device_manager = await subscriber.async_get_device_manager()
     except ApiException as err:
-        if DATA_NEST_UNAVAILABLE not in hass.data[DOMAIN]:
-            _LOGGER.error("Device manager error: %s", err)
-            hass.data[DOMAIN][DATA_NEST_UNAVAILABLE] = True
         subscriber.stop_async()
-        raise ConfigEntryNotReady from err
+        raise ConfigEntryNotReady(f"Device manager error: {str(err)}") from err
 
-    hass.data[DOMAIN].pop(DATA_NEST_UNAVAILABLE, None)
-    hass.data[DOMAIN][DATA_SUBSCRIBER] = subscriber
+    hass.data[DOMAIN][entry.entry_id] = {
+        DATA_SUBSCRIBER: subscriber,
+        DATA_DEVICE_MANAGER: device_manager,
+    }
 
-    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
+    await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
 
     return True
 
 
+async def async_import_config(hass: HomeAssistant, entry: ConfigEntry) -> None:
+    """Attempt to import configuration.yaml settings."""
+    config = hass.data[DOMAIN][DATA_NEST_CONFIG]
+    new_data = {
+        CONF_PROJECT_ID: config[CONF_PROJECT_ID],
+        **entry.data,
+    }
+    if CONF_SUBSCRIBER_ID not in entry.data:
+        if CONF_SUBSCRIBER_ID not in config:
+            raise ValueError("Configuration option 'subscriber_id' missing")
+        new_data.update(
+            {
+                CONF_SUBSCRIBER_ID: config[CONF_SUBSCRIBER_ID],
+                CONF_SUBSCRIBER_ID_IMPORTED: True,  # Don't delete user managed subscriber
+            }
+        )
+    hass.config_entries.async_update_entry(
+        entry, data=new_data, unique_id=new_data[CONF_PROJECT_ID]
+    )
+
+    if entry.data["auth_implementation"] == INSTALLED_AUTH_DOMAIN:
+        # App Auth credentials have been deprecated and must be re-created
+        # by the user in the config flow
+        async_create_issue(
+            hass,
+            DOMAIN,
+            "removed_app_auth",
+            is_fixable=False,
+            severity=IssueSeverity.ERROR,
+            translation_key="removed_app_auth",
+            translation_placeholders={
+                "more_info_url": "https://www.home-assistant.io/more-info/nest-auth-deprecation",
+                "documentation_url": "https://www.home-assistant.io/integrations/nest/",
+            },
+        )
+        raise ConfigEntryAuthFailed(
+            "Google has deprecated App Auth credentials, and the integration "
+            "must be reconfigured in the UI to restore access to Nest Devices."
+        )
+
+    if entry.data["auth_implementation"] == WEB_AUTH_DOMAIN:
+        await async_import_client_credential(
+            hass,
+            DOMAIN,
+            ClientCredential(
+                config[CONF_CLIENT_ID],
+                config[CONF_CLIENT_SECRET],
+            ),
+            WEB_AUTH_DOMAIN,
+        )
+
+    async_create_issue(
+        hass,
+        DOMAIN,
+        "deprecated_yaml",
+        breaks_in_ha_version="2022.10.0",
+        is_fixable=False,
+        severity=IssueSeverity.WARNING,
+        translation_key="deprecated_yaml",
+    )
+
+
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
     if DATA_SDM not in entry.data:
         # Legacy API
         return True
     _LOGGER.debug("Stopping nest subscriber")
-    subscriber = hass.data[DOMAIN][DATA_SUBSCRIBER]
+    subscriber = hass.data[DOMAIN][entry.entry_id][DATA_SUBSCRIBER]
     subscriber.stop_async()
     unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if unload_ok:
-        hass.data[DOMAIN].pop(DATA_SUBSCRIBER)
-        hass.data[DOMAIN].pop(DATA_NEST_UNAVAILABLE, None)
+        hass.data[DOMAIN].pop(entry.entry_id)
 
     return unload_ok
 
 
 async def async_remove_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
     """Handle removal of pubsub subscriptions created during config flow."""
-    if DATA_SDM not in entry.data or CONF_SUBSCRIBER_ID not in entry.data:
+    if (
+        DATA_SDM not in entry.data
+        or CONF_SUBSCRIBER_ID not in entry.data
+        or CONF_SUBSCRIBER_ID_IMPORTED in entry.data
+    ):
         return
 
     subscriber = await api.new_subscriber(hass, entry)
@@ -266,12 +353,12 @@ class NestEventViewBase(HomeAssistantView, ABC):
     ) -> web.StreamResponse:
         """Start a GET request."""
         user = request[KEY_HASS_USER]
-        entity_registry = await self.hass.helpers.entity_registry.async_get_registry()
+        entity_registry = er.async_get(self.hass)
         for entry in async_entries_for_device(entity_registry, device_id):
             if not user.permissions.check_entity(entry.entity_id, POLICY_READ):
                 raise Unauthorized(entity_id=entry.entity_id)
 
-        devices = await get_media_source_devices(self.hass)
+        devices = async_get_media_source_devices(self.hass)
         if not (nest_device := devices.get(device_id)):
             return self._json_error(
                 f"No Nest Device found for '{device_id}'", HTTPStatus.NOT_FOUND
