@@ -1,7 +1,7 @@
 """Class for helpers and communication with the OverKiz API."""
 from __future__ import annotations
 
-from typing import Any
+from typing import Any, cast
 from urllib.parse import urlparse
 
 from pyoverkiz.enums import OverkizCommand, Protocol
@@ -37,6 +37,10 @@ class OverkizExecutor:
         """Return Overkiz device linked to this entity."""
         return self.coordinator.data[self.device_url]
 
+    def linked_device(self, index: int) -> Device:
+        """Return Overkiz device sharing the same base url."""
+        return self.coordinator.data[f"{self.base_device_url}#{index}"]
+
     def select_command(self, *commands: str) -> str | None:
         """Select first existing command in a list of commands."""
         existing_commands = self.device.definition.commands
@@ -68,17 +72,18 @@ class OverkizExecutor:
 
     async def async_execute_command(self, command_name: str, *args: Any) -> None:
         """Execute device command in async context."""
+        parameters = [arg for arg in args if arg is not None]
         # Set the execution duration to 0 seconds for RTS devices on supported commands
         # Default execution duration is 30 seconds and will block consecutive commands
         if (
             self.device.protocol == Protocol.RTS
             and command_name not in COMMANDS_WITHOUT_DELAY
         ):
-            args = args + (0,)
+            parameters.append(0)
 
         exec_id = await self.coordinator.client.execute_command(
             self.device.device_url,
-            Command(command_name, list(args)),
+            Command(command_name, parameters),
             "Home Assistant",
         )
 
@@ -113,7 +118,9 @@ class OverkizExecutor:
             return True
 
         # Retrieve executions initiated outside Home Assistant via API
-        executions = await self.coordinator.client.get_current_executions()
+        executions = cast(Any, await self.coordinator.client.get_current_executions())
+        # executions.action_group is typed incorrectly in the upstream library
+        # or the below code is incorrect.
         exec_id = next(
             (
                 execution.id
