@@ -14,11 +14,13 @@ import re
 from statistics import mean
 from typing import TYPE_CHECKING, Any, Literal, overload
 
-from sqlalchemy import bindparam, func
+from sqlalchemy import bindparam, func, lambda_stmt, select
+from sqlalchemy.engine.row import Row
 from sqlalchemy.exc import SQLAlchemyError, StatementError
-from sqlalchemy.ext import baked
 from sqlalchemy.orm.session import Session
 from sqlalchemy.sql.expression import literal_column, true
+from sqlalchemy.sql.lambdas import StatementLambdaElement
+from sqlalchemy.sql.selectable import Subquery
 import voluptuous as vol
 
 from homeassistant.const import (
@@ -27,30 +29,34 @@ from homeassistant.const import (
     VOLUME_CUBIC_FEET,
     VOLUME_CUBIC_METERS,
 )
-from homeassistant.core import Event, HomeAssistant, callback
+from homeassistant.core import Event, HomeAssistant, callback, valid_entity_id
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import entity_registry
 from homeassistant.helpers.json import JSONEncoder
 from homeassistant.helpers.storage import STORAGE_DIR
+from homeassistant.helpers.typing import UNDEFINED, UndefinedType
 import homeassistant.util.dt as dt_util
 import homeassistant.util.pressure as pressure_util
 import homeassistant.util.temperature as temperature_util
 from homeassistant.util.unit_system import UnitSystem
 import homeassistant.util.volume as volume_util
 
-from .const import DATA_INSTANCE, DOMAIN, MAX_ROWS_TO_PURGE
+from .const import DOMAIN, MAX_ROWS_TO_PURGE, SupportedDialect
+from .db_schema import Statistics, StatisticsMeta, StatisticsRuns, StatisticsShortTerm
 from .models import (
     StatisticData,
     StatisticMetaData,
     StatisticResult,
-    Statistics,
-    StatisticsMeta,
-    StatisticsRuns,
-    StatisticsShortTerm,
     process_timestamp,
     process_timestamp_to_utc_isoformat,
 )
-from .util import execute, retryable_database_job, session_scope
+from .util import (
+    execute,
+    execute_stmt_lambda_element,
+    get_instance,
+    retryable_database_job,
+    session_scope,
+)
 
 if TYPE_CHECKING:
     from . import Recorder
@@ -120,8 +126,6 @@ QUERY_STATISTIC_META_ID = [
     StatisticsMeta.statistic_id,
 ]
 
-STATISTICS_BAKERY = "recorder_statistics_bakery"
-
 
 # Convert pressure, temperature and volume statistics from the normalized unit used for
 # statistics to the unit configured by the user
@@ -203,20 +207,12 @@ class ValidationIssue:
 
 def async_setup(hass: HomeAssistant) -> None:
     """Set up the history hooks."""
-    hass.data[STATISTICS_BAKERY] = baked.bakery()
 
-    def _entity_id_changed(event: Event) -> None:
-        """Handle entity_id changed."""
-        old_entity_id = event.data["old_entity_id"]
-        entity_id = event.data["entity_id"]
-        with session_scope(hass=hass) as session:
-            session.query(StatisticsMeta).filter(
-                (StatisticsMeta.statistic_id == old_entity_id)
-                & (StatisticsMeta.source == DOMAIN)
-            ).update({StatisticsMeta.statistic_id: entity_id})
-
-    async def _async_entity_id_changed(event: Event) -> None:
-        await hass.data[DATA_INSTANCE].async_add_executor_job(_entity_id_changed, event)
+    @callback
+    def _async_entity_id_changed(event: Event) -> None:
+        get_instance(hass).async_update_statistics_metadata(
+            event.data["old_entity_id"], new_statistic_id=event.data["entity_id"]
+        )
 
     @callback
     def entity_registry_changed_filter(event: Event) -> bool:
@@ -271,12 +267,14 @@ def _update_or_add_metadata(
     if (
         old_metadata["has_mean"] != new_metadata["has_mean"]
         or old_metadata["has_sum"] != new_metadata["has_sum"]
+        or old_metadata["name"] != new_metadata["name"]
         or old_metadata["unit_of_measurement"] != new_metadata["unit_of_measurement"]
     ):
         session.query(StatisticsMeta).filter_by(statistic_id=statistic_id).update(
             {
                 StatisticsMeta.has_mean: new_metadata["has_mean"],
                 StatisticsMeta.has_sum: new_metadata["has_sum"],
+                StatisticsMeta.name: new_metadata["name"],
                 StatisticsMeta.unit_of_measurement: new_metadata["unit_of_measurement"],
             },
             synchronize_session=False,
@@ -377,7 +375,7 @@ def _delete_duplicates_from_table(
     return (total_deleted_rows, all_non_identical_duplicates)
 
 
-def delete_duplicates(instance: Recorder, session: Session) -> None:
+def delete_statistics_duplicates(hass: HomeAssistant, session: Session) -> None:
     """Identify and delete duplicated statistics.
 
     A backup will be made of duplicated statistics before it is deleted.
@@ -391,7 +389,7 @@ def delete_duplicates(instance: Recorder, session: Session) -> None:
     if non_identical_duplicates:
         isotime = dt_util.utcnow().isoformat()
         backup_file_name = f"deleted_statistics.{isotime}.json"
-        backup_path = instance.hass.config.path(STORAGE_DIR, backup_file_name)
+        backup_path = hass.config.path(STORAGE_DIR, backup_file_name)
 
         os.makedirs(os.path.dirname(backup_path), exist_ok=True)
         with open(backup_path, "w", encoding="utf8") as backup_file:
@@ -415,11 +413,87 @@ def delete_duplicates(instance: Recorder, session: Session) -> None:
     )
     if deleted_short_term_statistics_rows:
         _LOGGER.warning(
-            "Deleted duplicated short term statistic rows, please report at "
-            'https://github.com/home-assistant/core/issues?q=is%%3Aissue+label%%3A"integration%%3A+recorder"+'
+            "Deleted duplicated short term statistic rows, please report at %s",
+            "https://github.com/home-assistant/core/issues?q=is%3Aopen+is%3Aissue+label%3A%22integration%3A+recorder%22",
         )
 
 
+def _find_statistics_meta_duplicates(session: Session) -> list[int]:
+    """Find duplicated statistics_meta."""
+    subquery = (
+        session.query(
+            StatisticsMeta.statistic_id,
+            literal_column("1").label("is_duplicate"),
+        )
+        .group_by(StatisticsMeta.statistic_id)
+        .having(func.count() > 1)
+        .subquery()
+    )
+    query = (
+        session.query(StatisticsMeta)
+        .outerjoin(
+            subquery,
+            (subquery.c.statistic_id == StatisticsMeta.statistic_id),
+        )
+        .filter(subquery.c.is_duplicate == 1)
+        .order_by(StatisticsMeta.statistic_id, StatisticsMeta.id.desc())
+        .limit(1000 * MAX_ROWS_TO_PURGE)
+    )
+    duplicates = execute(query)
+    statistic_id = None
+    duplicate_ids: list[int] = []
+
+    if not duplicates:
+        return duplicate_ids
+
+    for duplicate in duplicates:
+        if statistic_id != duplicate.statistic_id:
+            statistic_id = duplicate.statistic_id
+            continue
+        duplicate_ids.append(duplicate.id)
+
+    return duplicate_ids
+
+
+def _delete_statistics_meta_duplicates(session: Session) -> int:
+    """Identify and delete duplicated statistics from a specified table."""
+    total_deleted_rows = 0
+    while True:
+        duplicate_ids = _find_statistics_meta_duplicates(session)
+        if not duplicate_ids:
+            break
+        for i in range(0, len(duplicate_ids), MAX_ROWS_TO_PURGE):
+            deleted_rows = (
+                session.query(StatisticsMeta)
+                .filter(StatisticsMeta.id.in_(duplicate_ids[i : i + MAX_ROWS_TO_PURGE]))
+                .delete(synchronize_session=False)
+            )
+            total_deleted_rows += deleted_rows
+    return total_deleted_rows
+
+
+def delete_statistics_meta_duplicates(session: Session) -> None:
+    """Identify and delete duplicated statistics_meta."""
+    deleted_statistics_rows = _delete_statistics_meta_duplicates(session)
+    if deleted_statistics_rows:
+        _LOGGER.info(
+            "Deleted %s duplicated statistics_meta rows", deleted_statistics_rows
+        )
+
+
+def _compile_hourly_statistics_summary_mean_stmt(
+    start_time: datetime, end_time: datetime
+) -> StatementLambdaElement:
+    """Generate the summary mean statement for hourly statistics."""
+    return lambda_stmt(
+        lambda: select(*QUERY_STATISTICS_SUMMARY_MEAN)
+        .filter(StatisticsShortTerm.start >= start_time)
+        .filter(StatisticsShortTerm.start < end_time)
+        .group_by(StatisticsShortTerm.metadata_id)
+        .order_by(StatisticsShortTerm.metadata_id)
+    )
+
+
 def compile_hourly_statistics(
     instance: Recorder, session: Session, start: datetime
 ) -> None:
@@ -434,20 +508,8 @@ def compile_hourly_statistics(
 
     # Compute last hour's average, min, max
     summary: dict[str, StatisticData] = {}
-    baked_query = instance.hass.data[STATISTICS_BAKERY](
-        lambda session: session.query(*QUERY_STATISTICS_SUMMARY_MEAN)
-    )
-
-    baked_query += lambda q: q.filter(
-        StatisticsShortTerm.start >= bindparam("start_time")
-    )
-    baked_query += lambda q: q.filter(StatisticsShortTerm.start < bindparam("end_time"))
-    baked_query += lambda q: q.group_by(StatisticsShortTerm.metadata_id)
-    baked_query += lambda q: q.order_by(StatisticsShortTerm.metadata_id)
-
-    stats = execute(
-        baked_query(session).params(start_time=start_time, end_time=end_time)
-    )
+    stmt = _compile_hourly_statistics_summary_mean_stmt(start_time, end_time)
+    stats = execute_stmt_lambda_element(session, stmt)
 
     if stats:
         for stat in stats:
@@ -460,81 +522,37 @@ def compile_hourly_statistics(
             }
 
     # Get last hour's last sum
-    if instance._db_supports_row_number:  # pylint: disable=[protected-access]
-        subquery = (
-            session.query(*QUERY_STATISTICS_SUMMARY_SUM)
-            .filter(StatisticsShortTerm.start >= bindparam("start_time"))
-            .filter(StatisticsShortTerm.start < bindparam("end_time"))
-            .subquery()
-        )
-        query = (
-            session.query(subquery)
-            .filter(subquery.c.rownum == 1)
-            .order_by(subquery.c.metadata_id)
-        )
-        stats = execute(query.params(start_time=start_time, end_time=end_time))
+    subquery = (
+        session.query(*QUERY_STATISTICS_SUMMARY_SUM)
+        .filter(StatisticsShortTerm.start >= bindparam("start_time"))
+        .filter(StatisticsShortTerm.start < bindparam("end_time"))
+        .subquery()
+    )
+    query = (
+        session.query(subquery)
+        .filter(subquery.c.rownum == 1)
+        .order_by(subquery.c.metadata_id)
+    )
+    stats = execute(query.params(start_time=start_time, end_time=end_time))
 
-        if stats:
-            for stat in stats:
-                metadata_id, start, last_reset, state, _sum, _ = stat
-                if metadata_id in summary:
-                    summary[metadata_id].update(
-                        {
-                            "last_reset": process_timestamp(last_reset),
-                            "state": state,
-                            "sum": _sum,
-                        }
-                    )
-                else:
-                    summary[metadata_id] = {
-                        "start": start_time,
-                        "last_reset": process_timestamp(last_reset),
-                        "state": state,
-                        "sum": _sum,
-                    }
-    else:
-        baked_query = instance.hass.data[STATISTICS_BAKERY](
-            lambda session: session.query(*QUERY_STATISTICS_SUMMARY_SUM_LEGACY)
-        )
-
-        baked_query += lambda q: q.filter(
-            StatisticsShortTerm.start >= bindparam("start_time")
-        )
-        baked_query += lambda q: q.filter(
-            StatisticsShortTerm.start < bindparam("end_time")
-        )
-        baked_query += lambda q: q.order_by(
-            StatisticsShortTerm.metadata_id, StatisticsShortTerm.start.desc()
-        )
-
-        stats = execute(
-            baked_query(session).params(start_time=start_time, end_time=end_time)
-        )
-
-        if stats:
-            for metadata_id, group in groupby(stats, lambda stat: stat["metadata_id"]):  # type: ignore[no-any-return]
-                (
-                    metadata_id,
-                    last_reset,
-                    state,
-                    _sum,
-                ) = next(group)
-                if metadata_id in summary:
-                    summary[metadata_id].update(
-                        {
-                            "start": start_time,
-                            "last_reset": process_timestamp(last_reset),
-                            "state": state,
-                            "sum": _sum,
-                        }
-                    )
-                else:
-                    summary[metadata_id] = {
-                        "start": start_time,
+    if stats:
+        for stat in stats:
+            metadata_id, start, last_reset, state, _sum, _ = stat
+            if metadata_id in summary:
+                summary[metadata_id].update(
+                    {
                         "last_reset": process_timestamp(last_reset),
                         "state": state,
                         "sum": _sum,
                     }
+                )
+            else:
+                summary[metadata_id] = {
+                    "start": start_time,
+                    "last_reset": process_timestamp(last_reset),
+                    "state": state,
+                    "sum": _sum,
+                }
 
     # Insert compiled hourly statistics in the database
     for metadata_id, stat in summary.items():
@@ -551,7 +569,7 @@ def compile_statistics(instance: Recorder, start: datetime) -> bool:
     end = start + timedelta(minutes=5)
 
     # Return if we already have 5-minute statistics for the requested period
-    with session_scope(session=instance.get_session()) as session:  # type: ignore[misc]
+    with session_scope(session=instance.get_session()) as session:
         if session.query(StatisticsRuns).filter_by(start=start).first():
             _LOGGER.debug("Statistics already compiled for %s-%s", start, end)
             return True
@@ -560,7 +578,7 @@ def compile_statistics(instance: Recorder, start: datetime) -> bool:
     platform_stats: list[StatisticResult] = []
     current_metadata: dict[str, tuple[int, StatisticMetaData]] = {}
     # Collect statistics from all platforms implementing support
-    for domain, platform in instance.hass.data[DOMAIN].items():
+    for domain, platform in instance.hass.data[DOMAIN].recorder_platforms.items():
         if not hasattr(platform, "compile_statistics"):
             continue
         compiled: PlatformCompiledStatistics = platform.compile_statistics(
@@ -578,7 +596,7 @@ def compile_statistics(instance: Recorder, start: datetime) -> bool:
 
     # Insert collected statistics in the database
     with session_scope(
-        session=instance.get_session(),  # type: ignore[misc]
+        session=instance.get_session(),
         exception_filter=_filter_unique_constraint_integrity_error(instance),
     ) as session:
         for stats in platform_stats:
@@ -669,6 +687,24 @@ def _update_statistics(
         )
 
 
+def _generate_get_metadata_stmt(
+    statistic_ids: list[str] | tuple[str] | None = None,
+    statistic_type: Literal["mean"] | Literal["sum"] | None = None,
+    statistic_source: str | None = None,
+) -> StatementLambdaElement:
+    """Generate a statement to fetch metadata."""
+    stmt = lambda_stmt(lambda: select(*QUERY_STATISTIC_META))
+    if statistic_ids is not None:
+        stmt += lambda q: q.where(StatisticsMeta.statistic_id.in_(statistic_ids))
+    if statistic_source is not None:
+        stmt += lambda q: q.where(StatisticsMeta.source == statistic_source)
+    if statistic_type == "mean":
+        stmt += lambda q: q.where(StatisticsMeta.has_mean == true())
+    elif statistic_type == "sum":
+        stmt += lambda q: q.where(StatisticsMeta.has_sum == true())
+    return stmt
+
+
 def get_metadata_with_session(
     hass: HomeAssistant,
     session: Session,
@@ -686,26 +722,8 @@ def get_metadata_with_session(
     """
 
     # Fetch metatadata from the database
-    baked_query = hass.data[STATISTICS_BAKERY](
-        lambda session: session.query(*QUERY_STATISTIC_META)
-    )
-    if statistic_ids is not None:
-        baked_query += lambda q: q.filter(
-            StatisticsMeta.statistic_id.in_(bindparam("statistic_ids"))
-        )
-    if statistic_source is not None:
-        baked_query += lambda q: q.filter(
-            StatisticsMeta.source == bindparam("statistic_source")
-        )
-    if statistic_type == "mean":
-        baked_query += lambda q: q.filter(StatisticsMeta.has_mean == true())
-    elif statistic_type == "sum":
-        baked_query += lambda q: q.filter(StatisticsMeta.has_sum == true())
-    result = execute(
-        baked_query(session).params(
-            statistic_ids=statistic_ids, statistic_source=statistic_source
-        )
-    )
+    stmt = _generate_get_metadata_stmt(statistic_ids, statistic_type, statistic_source)
+    result = execute_stmt_lambda_element(session, stmt)
     if not result:
         return {}
 
@@ -768,20 +786,33 @@ def _configured_unit(unit: str | None, units: UnitSystem) -> str | None:
 
 def clear_statistics(instance: Recorder, statistic_ids: list[str]) -> None:
     """Clear statistics for a list of statistic_ids."""
-    with session_scope(session=instance.get_session()) as session:  # type: ignore[misc]
+    with session_scope(session=instance.get_session()) as session:
         session.query(StatisticsMeta).filter(
             StatisticsMeta.statistic_id.in_(statistic_ids)
         ).delete(synchronize_session=False)
 
 
 def update_statistics_metadata(
-    instance: Recorder, statistic_id: str, unit_of_measurement: str | None
+    instance: Recorder,
+    statistic_id: str,
+    new_statistic_id: str | None | UndefinedType,
+    new_unit_of_measurement: str | None | UndefinedType,
 ) -> None:
     """Update statistics metadata for a statistic_id."""
-    with session_scope(session=instance.get_session()) as session:  # type: ignore[misc]
-        session.query(StatisticsMeta).filter(
-            StatisticsMeta.statistic_id == statistic_id
-        ).update({StatisticsMeta.unit_of_measurement: unit_of_measurement})
+    if new_unit_of_measurement is not UNDEFINED:
+        with session_scope(session=instance.get_session()) as session:
+            session.query(StatisticsMeta).filter(
+                StatisticsMeta.statistic_id == statistic_id
+            ).update({StatisticsMeta.unit_of_measurement: new_unit_of_measurement})
+    if new_statistic_id is not UNDEFINED:
+        with session_scope(
+            session=instance.get_session(),
+            exception_filter=_filter_unique_constraint_integrity_error(instance),
+        ) as session:
+            session.query(StatisticsMeta).filter(
+                (StatisticsMeta.statistic_id == statistic_id)
+                & (StatisticsMeta.source == DOMAIN)
+            ).update({StatisticsMeta.statistic_id: new_statistic_id})
 
 
 def list_statistic_ids(
@@ -795,48 +826,54 @@ def list_statistic_ids(
     a recorder platform for statistic_ids which will be added in the next statistics
     period.
     """
-    units = hass.config.units
     result = {}
 
+    def _display_unit(hass: HomeAssistant, unit: str | None) -> str | None:
+        if unit is None:
+            return None
+        return _configured_unit(unit, hass.config.units)
+
     # Query the database
     with session_scope(hass=hass) as session:
         metadata = get_metadata_with_session(
             hass, session, statistic_type=statistic_type, statistic_ids=statistic_ids
         )
 
-        for _, meta in metadata.values():
-            if (unit := meta["unit_of_measurement"]) is not None:
-                # Display unit according to user settings
-                unit = _configured_unit(unit, units)
-            meta["unit_of_measurement"] = unit
-
         result = {
             meta["statistic_id"]: {
                 "has_mean": meta["has_mean"],
                 "has_sum": meta["has_sum"],
                 "name": meta["name"],
                 "source": meta["source"],
+                "display_unit_of_measurement": _display_unit(
+                    hass, meta["unit_of_measurement"]
+                ),
                 "unit_of_measurement": meta["unit_of_measurement"],
             }
             for _, meta in metadata.values()
         }
 
     # Query all integrations with a registered recorder platform
-    for platform in hass.data[DOMAIN].values():
+    for platform in hass.data[DOMAIN].recorder_platforms.values():
         if not hasattr(platform, "list_statistic_ids"):
             continue
         platform_statistic_ids = platform.list_statistic_ids(
             hass, statistic_ids=statistic_ids, statistic_type=statistic_type
         )
 
-        for statistic_id, info in platform_statistic_ids.items():
-            if (unit := info["unit_of_measurement"]) is not None:
-                # Display unit according to user settings
-                unit = _configured_unit(unit, units)
-            platform_statistic_ids[statistic_id]["unit_of_measurement"] = unit
-
-        for key, value in platform_statistic_ids.items():
-            result.setdefault(key, value)
+        for key, meta in platform_statistic_ids.items():
+            if key in result:
+                continue
+            result[key] = {
+                "has_mean": meta["has_mean"],
+                "has_sum": meta["has_sum"],
+                "name": meta["name"],
+                "source": meta["source"],
+                "display_unit_of_measurement": _display_unit(
+                    hass, meta["unit_of_measurement"]
+                ),
+                "unit_of_measurement": meta["unit_of_measurement"],
+            }
 
     # Return a list of statistic_id + metadata
     return [
@@ -846,37 +883,13 @@ def list_statistic_ids(
             "has_sum": info["has_sum"],
             "name": info.get("name"),
             "source": info["source"],
-            "unit_of_measurement": info["unit_of_measurement"],
+            "display_unit_of_measurement": info["display_unit_of_measurement"],
+            "statistics_unit_of_measurement": info["unit_of_measurement"],
         }
         for _id, info in result.items()
     ]
 
 
-def _statistics_during_period_query(
-    hass: HomeAssistant,
-    end_time: datetime | None,
-    statistic_ids: list[str] | None,
-    baked_query: baked.BakedQuery,
-    table: type[Statistics | StatisticsShortTerm],
-) -> Callable:
-    """Prepare a database query for statistics during a given period.
-
-    This prepares a baked query, so we don't insert the parameters yet.
-    """
-    baked_query += lambda q: q.filter(table.start >= bindparam("start_time"))
-
-    if end_time is not None:
-        baked_query += lambda q: q.filter(table.start < bindparam("end_time"))
-
-    if statistic_ids is not None:
-        baked_query += lambda q: q.filter(
-            table.metadata_id.in_(bindparam("metadata_ids"))
-        )
-
-    baked_query += lambda q: q.order_by(table.metadata_id, table.start)
-    return baked_query  # type: ignore[no-any-return]
-
-
 def _reduce_statistics(
     stats: dict[str, list[dict[str, Any]]],
     same_period: Callable[[datetime, datetime], bool],
@@ -975,6 +988,50 @@ def _reduce_statistics_per_month(
     return _reduce_statistics(stats, same_month, month_start_end, timedelta(days=31))
 
 
+def _statistics_during_period_stmt(
+    start_time: datetime,
+    end_time: datetime | None,
+    metadata_ids: list[int] | None,
+) -> StatementLambdaElement:
+    """Prepare a database query for statistics during a given period.
+
+    This prepares a lambda_stmt query, so we don't insert the parameters yet.
+    """
+    stmt = lambda_stmt(
+        lambda: select(*QUERY_STATISTICS).filter(Statistics.start >= start_time)
+    )
+    if end_time is not None:
+        stmt += lambda q: q.filter(Statistics.start < end_time)
+    if metadata_ids:
+        stmt += lambda q: q.filter(Statistics.metadata_id.in_(metadata_ids))
+    stmt += lambda q: q.order_by(Statistics.metadata_id, Statistics.start)
+    return stmt
+
+
+def _statistics_during_period_stmt_short_term(
+    start_time: datetime,
+    end_time: datetime | None,
+    metadata_ids: list[int] | None,
+) -> StatementLambdaElement:
+    """Prepare a database query for short term statistics during a given period.
+
+    This prepares a lambda_stmt query, so we don't insert the parameters yet.
+    """
+    stmt = lambda_stmt(
+        lambda: select(*QUERY_STATISTICS_SHORT_TERM).filter(
+            StatisticsShortTerm.start >= start_time
+        )
+    )
+    if end_time is not None:
+        stmt += lambda q: q.filter(StatisticsShortTerm.start < end_time)
+    if metadata_ids:
+        stmt += lambda q: q.filter(StatisticsShortTerm.metadata_id.in_(metadata_ids))
+    stmt += lambda q: q.order_by(
+        StatisticsShortTerm.metadata_id, StatisticsShortTerm.start
+    )
+    return stmt
+
+
 def statistics_during_period(
     hass: HomeAssistant,
     start_time: datetime,
@@ -999,25 +1056,16 @@ def statistics_during_period(
         if statistic_ids is not None:
             metadata_ids = [metadata_id for metadata_id, _ in metadata.values()]
 
-        bakery = hass.data[STATISTICS_BAKERY]
         if period == "5minute":
-            baked_query = bakery(
-                lambda session: session.query(*QUERY_STATISTICS_SHORT_TERM)
-            )
             table = StatisticsShortTerm
-        else:
-            baked_query = bakery(lambda session: session.query(*QUERY_STATISTICS))
-            table = Statistics
-
-        baked_query = _statistics_during_period_query(
-            hass, end_time, statistic_ids, baked_query, table
-        )
-
-        stats = execute(
-            baked_query(session).params(
-                start_time=start_time, end_time=end_time, metadata_ids=metadata_ids
+            stmt = _statistics_during_period_stmt_short_term(
+                start_time, end_time, metadata_ids
             )
-        )
+        else:
+            table = Statistics
+            stmt = _statistics_during_period_stmt(start_time, end_time, metadata_ids)
+        stats = execute_stmt_lambda_element(session, stmt)
+
         if not stats:
             return {}
         # Return statistics combined with metadata
@@ -1044,6 +1092,32 @@ def statistics_during_period(
         return _reduce_statistics_per_month(result)
 
 
+def _get_last_statistics_stmt(
+    metadata_id: int,
+    number_of_stats: int,
+) -> StatementLambdaElement:
+    """Generate a statement for number_of_stats statistics for a given statistic_id."""
+    return lambda_stmt(
+        lambda: select(*QUERY_STATISTICS)
+        .filter_by(metadata_id=metadata_id)
+        .order_by(Statistics.metadata_id, Statistics.start.desc())
+        .limit(number_of_stats)
+    )
+
+
+def _get_last_statistics_short_term_stmt(
+    metadata_id: int,
+    number_of_stats: int,
+) -> StatementLambdaElement:
+    """Generate a statement for number_of_stats short term statistics for a given statistic_id."""
+    return lambda_stmt(
+        lambda: select(*QUERY_STATISTICS_SHORT_TERM)
+        .filter_by(metadata_id=metadata_id)
+        .order_by(StatisticsShortTerm.metadata_id, StatisticsShortTerm.start.desc())
+        .limit(number_of_stats)
+    )
+
+
 def _get_last_statistics(
     hass: HomeAssistant,
     number_of_stats: int,
@@ -1058,27 +1132,13 @@ def _get_last_statistics(
         metadata = get_metadata_with_session(hass, session, statistic_ids=statistic_ids)
         if not metadata:
             return {}
-
-        bakery = hass.data[STATISTICS_BAKERY]
-        if table == StatisticsShortTerm:
-            baked_query = bakery(
-                lambda session: session.query(*QUERY_STATISTICS_SHORT_TERM)
-            )
-        else:
-            baked_query = bakery(lambda session: session.query(*QUERY_STATISTICS))
-
-        baked_query += lambda q: q.filter_by(metadata_id=bindparam("metadata_id"))
         metadata_id = metadata[statistic_id][0]
+        if table == Statistics:
+            stmt = _get_last_statistics_stmt(metadata_id, number_of_stats)
+        else:
+            stmt = _get_last_statistics_short_term_stmt(metadata_id, number_of_stats)
+        stats = execute_stmt_lambda_element(session, stmt)
 
-        baked_query += lambda q: q.order_by(table.metadata_id, table.start.desc())
-
-        baked_query += lambda q: q.limit(bindparam("number_of_stats"))
-
-        stats = execute(
-            baked_query(session).params(
-                number_of_stats=number_of_stats, metadata_id=metadata_id
-            )
-        )
         if not stats:
             return {}
 
@@ -1113,14 +1173,41 @@ def get_last_short_term_statistics(
     )
 
 
+def _generate_most_recent_statistic_row(metadata_ids: list[int]) -> Subquery:
+    """Generate the subquery to find the most recent statistic row."""
+    return (
+        select(
+            StatisticsShortTerm.metadata_id,
+            func.max(StatisticsShortTerm.start).label("start_max"),
+        )
+        .where(StatisticsShortTerm.metadata_id.in_(metadata_ids))
+        .group_by(StatisticsShortTerm.metadata_id)
+    ).subquery()
+
+
+def _latest_short_term_statistics_stmt(
+    metadata_ids: list[int],
+) -> StatementLambdaElement:
+    """Create the statement for finding the latest short term stat rows."""
+    stmt = lambda_stmt(lambda: select(*QUERY_STATISTICS_SHORT_TERM))
+    most_recent_statistic_row = _generate_most_recent_statistic_row(metadata_ids)
+    stmt += lambda s: s.join(
+        most_recent_statistic_row,
+        (
+            StatisticsShortTerm.metadata_id  # pylint: disable=comparison-with-callable
+            == most_recent_statistic_row.c.metadata_id
+        )
+        & (StatisticsShortTerm.start == most_recent_statistic_row.c.start_max),
+    )
+    return stmt
+
+
 def get_latest_short_term_statistics(
     hass: HomeAssistant,
     statistic_ids: list[str],
     metadata: dict[str, tuple[int, StatisticMetaData]] | None = None,
 ) -> dict[str, list[dict]]:
     """Return the latest short term statistics for a list of statistic_ids."""
-    # This function doesn't use a baked query, we instead rely on the
-    # "Transparent SQL Compilation Caching" feature introduced in SQLAlchemy 1.4
     with session_scope(hass=hass) as session:
         # Fetch metadata for the given statistic_ids
         if not metadata:
@@ -1134,24 +1221,8 @@ def get_latest_short_term_statistics(
             for statistic_id in statistic_ids
             if statistic_id in metadata
         ]
-        most_recent_statistic_row = (
-            session.query(
-                StatisticsShortTerm.metadata_id,
-                func.max(StatisticsShortTerm.start).label("start_max"),
-            )
-            .filter(StatisticsShortTerm.metadata_id.in_(metadata_ids))
-            .group_by(StatisticsShortTerm.metadata_id)
-        ).subquery()
-        stats = execute(
-            session.query(*QUERY_STATISTICS_SHORT_TERM).join(
-                most_recent_statistic_row,
-                (
-                    StatisticsShortTerm.metadata_id  # pylint: disable=comparison-with-callable
-                    == most_recent_statistic_row.c.metadata_id
-                )
-                & (StatisticsShortTerm.start == most_recent_statistic_row.c.start_max),
-            )
-        )
+        stmt = _latest_short_term_statistics_stmt(metadata_ids)
+        stats = execute_stmt_lambda_element(session, stmt)
         if not stats:
             return {}
 
@@ -1203,7 +1274,7 @@ def _statistics_at_time(
 def _sorted_statistics_to_dict(
     hass: HomeAssistant,
     session: Session,
-    stats: list,
+    stats: Iterable[Row],
     statistic_ids: list[str] | None,
     _metadata: dict[str, tuple[int, StatisticMetaData]],
     convert_units: bool,
@@ -1215,7 +1286,7 @@ def _sorted_statistics_to_dict(
     result: dict = defaultdict(list)
     units = hass.config.units
     metadata = dict(_metadata.values())
-    need_stat_at_start_time = set()
+    need_stat_at_start_time: set[int] = set()
     stats_at_start_time = {}
 
     def no_conversion(val: Any, _: Any) -> float | None:
@@ -1277,7 +1348,7 @@ def _sorted_statistics_to_dict(
 def validate_statistics(hass: HomeAssistant) -> dict[str, list[ValidationIssue]]:
     """Validate statistics."""
     platform_validation: dict[str, list[ValidationIssue]] = {}
-    for platform in hass.data[DOMAIN].values():
+    for platform in hass.data[DOMAIN].recorder_platforms.values():
         if not hasattr(platform, "validate_statistics"):
             continue
         platform_validation.update(platform.validate_statistics(hass))
@@ -1299,6 +1370,54 @@ def _statistics_exists(
     return result["id"] if result else None
 
 
+@callback
+def _async_import_statistics(
+    hass: HomeAssistant,
+    metadata: StatisticMetaData,
+    statistics: Iterable[StatisticData],
+) -> None:
+    """Validate timestamps and insert an import_statistics job in the recorder's queue."""
+    for statistic in statistics:
+        start = statistic["start"]
+        if start.tzinfo is None or start.tzinfo.utcoffset(start) is None:
+            raise HomeAssistantError("Naive timestamp")
+        if start.minute != 0 or start.second != 0 or start.microsecond != 0:
+            raise HomeAssistantError("Invalid timestamp")
+        statistic["start"] = dt_util.as_utc(start)
+
+        if "last_reset" in statistic and statistic["last_reset"] is not None:
+            last_reset = statistic["last_reset"]
+            if (
+                last_reset.tzinfo is None
+                or last_reset.tzinfo.utcoffset(last_reset) is None
+            ):
+                raise HomeAssistantError("Naive timestamp")
+            statistic["last_reset"] = dt_util.as_utc(last_reset)
+
+    # Insert job in recorder's queue
+    get_instance(hass).async_import_statistics(metadata, statistics)
+
+
+@callback
+def async_import_statistics(
+    hass: HomeAssistant,
+    metadata: StatisticMetaData,
+    statistics: Iterable[StatisticData],
+) -> None:
+    """Import hourly statistics from an internal source.
+
+    This inserts an import_statistics job in the recorder's queue.
+    """
+    if not valid_entity_id(metadata["statistic_id"]):
+        raise HomeAssistantError("Invalid statistic_id")
+
+    # The source must not be empty and must be aligned with the statistic_id
+    if not metadata["source"] or metadata["source"] != DOMAIN:
+        raise HomeAssistantError("Invalid source")
+
+    _async_import_statistics(hass, metadata, statistics)
+
+
 @callback
 def async_add_external_statistics(
     hass: HomeAssistant,
@@ -1307,7 +1426,7 @@ def async_add_external_statistics(
 ) -> None:
     """Add hourly statistics from an external source.
 
-    This inserts an add_external_statistics job in the recorder's queue.
+    This inserts an import_statistics job in the recorder's queue.
     """
     # The statistic_id has same limitations as an entity_id, but with a ':' as separator
     if not valid_statistic_id(metadata["statistic_id"]):
@@ -1318,16 +1437,7 @@ def async_add_external_statistics(
     if not metadata["source"] or metadata["source"] != domain:
         raise HomeAssistantError("Invalid source")
 
-    for statistic in statistics:
-        start = statistic["start"]
-        if start.tzinfo is None or start.tzinfo.utcoffset(start) is None:
-            raise HomeAssistantError("Naive timestamp")
-        if start.minute != 0 or start.second != 0 or start.microsecond != 0:
-            raise HomeAssistantError("Invalid timestamp")
-        statistic["start"] = dt_util.as_utc(start)
-
-    # Insert job in recorder's queue
-    hass.data[DATA_INSTANCE].async_external_statistics(metadata, statistics)
+    _async_import_statistics(hass, metadata, statistics)
 
 
 def _filter_unique_constraint_integrity_error(
@@ -1342,10 +1452,13 @@ def _filter_unique_constraint_integrity_error(
         dialect_name = instance.engine.dialect.name
 
         ignore = False
-        if dialect_name == "sqlite" and "UNIQUE constraint failed" in str(err):
+        if (
+            dialect_name == SupportedDialect.SQLITE
+            and "UNIQUE constraint failed" in str(err)
+        ):
             ignore = True
         if (
-            dialect_name == "postgresql"
+            dialect_name == SupportedDialect.POSTGRESQL
             and hasattr(err.orig, "pgcode")
             and err.orig.pgcode == "23505"
         ):
@@ -1357,8 +1470,8 @@ def _filter_unique_constraint_integrity_error(
 
         if ignore:
             _LOGGER.warning(
-                "Blocked attempt to insert duplicated statistic rows, please report at "
-                'https://github.com/home-assistant/core/issues?q=is%%3Aissue+label%%3A"integration%%3A+recorder"+',
+                "Blocked attempt to insert duplicated statistic rows, please report at %s",
+                "https://github.com/home-assistant/core/issues?q=is%3Aopen+is%3Aissue+label%3A%22integration%3A+recorder%22",
                 exc_info=err,
             )
 
@@ -1368,15 +1481,15 @@ def _filter_unique_constraint_integrity_error(
 
 
 @retryable_database_job("statistics")
-def add_external_statistics(
+def import_statistics(
     instance: Recorder,
     metadata: StatisticMetaData,
     statistics: Iterable[StatisticData],
 ) -> bool:
-    """Process an add_external_statistics job."""
+    """Process an import_statistics job."""
 
     with session_scope(
-        session=instance.get_session(),  # type: ignore[misc]
+        session=instance.get_session(),
         exception_filter=_filter_unique_constraint_integrity_error(instance),
     ) as session:
         old_metadata_dict = get_metadata_with_session(
@@ -1403,7 +1516,7 @@ def adjust_statistics(
 ) -> bool:
     """Process an add_statistics job."""
 
-    with session_scope(session=instance.get_session()) as session:  # type: ignore[misc]
+    with session_scope(session=instance.get_session()) as session:
         metadata = get_metadata_with_session(
             instance.hass, session, statistic_ids=(statistic_id,)
         )
