@@ -1,14 +1,20 @@
 """Config flow for Risco integration."""
+from __future__ import annotations
+
+from collections.abc import Mapping
 import logging
 
-from pyrisco import CannotConnectError, RiscoAPI, UnauthorizedError
+from pyrisco import CannotConnectError, RiscoCloud, RiscoLocal, UnauthorizedError
 import voluptuous as vol
 
 from homeassistant import config_entries, core
 from homeassistant.const import (
+    CONF_HOST,
     CONF_PASSWORD,
     CONF_PIN,
+    CONF_PORT,
     CONF_SCAN_INTERVAL,
+    CONF_TYPE,
     CONF_USERNAME,
     STATE_ALARM_ARMED_AWAY,
     STATE_ALARM_ARMED_CUSTOM_BYPASS,
@@ -25,12 +31,26 @@ from .const import (
     DEFAULT_OPTIONS,
     DOMAIN,
     RISCO_STATES,
+    TYPE_LOCAL,
 )
 
 _LOGGER = logging.getLogger(__name__)
 
 
-DATA_SCHEMA = vol.Schema({CONF_USERNAME: str, CONF_PASSWORD: str, CONF_PIN: str})
+CLOUD_SCHEMA = vol.Schema(
+    {
+        vol.Required(CONF_USERNAME): str,
+        vol.Required(CONF_PASSWORD): str,
+        vol.Required(CONF_PIN): str,
+    }
+)
+LOCAL_SCHEMA = vol.Schema(
+    {
+        vol.Required(CONF_HOST): str,
+        vol.Required(CONF_PORT, default=1000): int,
+        vol.Required(CONF_PIN): str,
+    }
+)
 HA_STATES = [
     STATE_ALARM_ARMED_AWAY,
     STATE_ALARM_ARMED_HOME,
@@ -39,12 +59,12 @@ HA_STATES = [
 ]
 
 
-async def validate_input(hass: core.HomeAssistant, data):
-    """Validate the user input allows us to connect.
+async def validate_cloud_input(hass: core.HomeAssistant, data) -> dict[str, str]:
+    """Validate the user input allows us to connect to Risco Cloud.
 
-    Data has the keys from DATA_SCHEMA with values provided by the user.
+    Data has the keys from CLOUD_SCHEMA with values provided by the user.
     """
-    risco = RiscoAPI(data[CONF_USERNAME], data[CONF_PASSWORD], data[CONF_PIN])
+    risco = RiscoCloud(data[CONF_USERNAME], data[CONF_PASSWORD], data[CONF_PIN])
 
     try:
         await risco.login(async_get_clientsession(hass))
@@ -54,27 +74,49 @@ async def validate_input(hass: core.HomeAssistant, data):
     return {"title": risco.site_name}
 
 
+async def validate_local_input(
+    hass: core.HomeAssistant, data: Mapping[str, str]
+) -> dict[str, str]:
+    """Validate the user input allows us to connect to a local panel.
+
+    Data has the keys from LOCAL_SCHEMA with values provided by the user.
+    """
+    risco = RiscoLocal(data[CONF_HOST], data[CONF_PORT], data[CONF_PIN])
+    await risco.connect()
+    site_id = risco.id
+    await risco.disconnect()
+    return {"title": site_id}
+
+
 class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
     """Handle a config flow for Risco."""
 
     VERSION = 1
-    CONNECTION_CLASS = config_entries.CONN_CLASS_CLOUD_POLL
 
     @staticmethod
     @core.callback
-    def async_get_options_flow(config_entry):
+    def async_get_options_flow(
+        config_entry: config_entries.ConfigEntry,
+    ) -> RiscoOptionsFlowHandler:
         """Define the config flow to handle options."""
         return RiscoOptionsFlowHandler(config_entry)
 
     async def async_step_user(self, user_input=None):
         """Handle the initial step."""
+        return self.async_show_menu(
+            step_id="user",
+            menu_options=["cloud", "local"],
+        )
+
+    async def async_step_cloud(self, user_input=None):
+        """Configure a cloud based alarm."""
         errors = {}
         if user_input is not None:
             await self.async_set_unique_id(user_input[CONF_USERNAME])
             self._abort_if_unique_id_configured()
 
             try:
-                info = await validate_input(self.hass, user_input)
+                info = await validate_cloud_input(self.hass, user_input)
             except CannotConnectError:
                 errors["base"] = "cannot_connect"
             except UnauthorizedError:
@@ -86,14 +128,39 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 return self.async_create_entry(title=info["title"], data=user_input)
 
         return self.async_show_form(
-            step_id="user", data_schema=DATA_SCHEMA, errors=errors
+            step_id="cloud", data_schema=CLOUD_SCHEMA, errors=errors
+        )
+
+    async def async_step_local(self, user_input=None):
+        """Configure a local based alarm."""
+        errors = {}
+        if user_input is not None:
+            try:
+                info = await validate_local_input(self.hass, user_input)
+            except CannotConnectError:
+                errors["base"] = "cannot_connect"
+            except UnauthorizedError:
+                errors["base"] = "invalid_auth"
+            except Exception:  # pylint: disable=broad-except
+                _LOGGER.exception("Unexpected exception")
+                errors["base"] = "unknown"
+            else:
+                await self.async_set_unique_id(info["title"])
+                self._abort_if_unique_id_configured()
+
+                return self.async_create_entry(
+                    title=info["title"], data={**user_input, **{CONF_TYPE: TYPE_LOCAL}}
+                )
+
+        return self.async_show_form(
+            step_id="local", data_schema=LOCAL_SCHEMA, errors=errors
         )
 
 
 class RiscoOptionsFlowHandler(config_entries.OptionsFlow):
     """Handle a Risco options flow."""
 
-    def __init__(self, config_entry):
+    def __init__(self, config_entry: config_entries.ConfigEntry) -> None:
         """Initialize."""
         self.config_entry = config_entry
         self._data = {**DEFAULT_OPTIONS, **config_entry.options}
