@@ -3,7 +3,7 @@ from __future__ import annotations
 
 import functools
 import numbers
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, TypeVar
 
 from homeassistant.components.climate.const import HVACAction
 from homeassistant.components.sensor import (
@@ -69,6 +69,13 @@ if TYPE_CHECKING:
     from .core.channels.base import ZigbeeChannel
     from .core.device import ZHADevice
 
+_SensorSelfT = TypeVar("_SensorSelfT", bound="Sensor")
+_BatterySelfT = TypeVar("_BatterySelfT", bound="Battery")
+_ThermostatHVACActionSelfT = TypeVar(
+    "_ThermostatHVACActionSelfT", bound="ThermostatHVACAction"
+)
+_RSSISensorSelfT = TypeVar("_RSSISensorSelfT", bound="RSSISensor")
+
 PARALLEL_UPDATES = 5
 
 BATTERY_SIZES = {
@@ -126,7 +133,7 @@ class Sensor(ZhaEntity, SensorEntity):
         unique_id: str,
         zha_device: ZHADevice,
         channels: list[ZigbeeChannel],
-        **kwargs,
+        **kwargs: Any,
     ) -> None:
         """Init this sensor."""
         super().__init__(unique_id, zha_device, channels, **kwargs)
@@ -134,12 +141,12 @@ class Sensor(ZhaEntity, SensorEntity):
 
     @classmethod
     def create_entity(
-        cls,
+        cls: type[_SensorSelfT],
         unique_id: str,
         zha_device: ZHADevice,
         channels: list[ZigbeeChannel],
-        **kwargs,
-    ) -> ZhaEntity | None:
+        **kwargs: Any,
+    ) -> _SensorSelfT | None:
         """Entity Factory.
 
         Return entity if it is a supported configuration, otherwise return None
@@ -176,7 +183,7 @@ class Sensor(ZhaEntity, SensorEntity):
         """Handle state update from channel."""
         self.async_write_ha_state()
 
-    def formatter(self, value: int) -> int | float:
+    def formatter(self, value: int) -> int | float | None:
         """Numeric pass-through formatter."""
         if self._decimals > 0:
             return round(
@@ -214,12 +221,12 @@ class Battery(Sensor):
 
     @classmethod
     def create_entity(
-        cls,
+        cls: type[_BatterySelfT],
         unique_id: str,
         zha_device: ZHADevice,
         channels: list[ZigbeeChannel],
-        **kwargs,
-    ) -> ZhaEntity | None:
+        **kwargs: Any,
+    ) -> _BatterySelfT | None:
         """Entity Factory.
 
         Unlike any other entity, PowerConfiguration cluster may not support
@@ -229,11 +236,11 @@ class Battery(Sensor):
         return cls(unique_id, zha_device, channels, **kwargs)
 
     @staticmethod
-    def formatter(value: int) -> int:  # pylint: disable=arguments-differ
+    def formatter(value: int) -> int | None:  # pylint: disable=arguments-differ
         """Return the state of the entity."""
         # per zcl specs battery percent is reported at 200% ¯\_(ツ)_/¯
         if not isinstance(value, numbers.Number) or value == -1:
-            return value
+            return None
         value = round(value / 2)
         return value
 
@@ -259,15 +266,11 @@ class ElectricalMeasurement(Sensor):
 
     SENSOR_ATTR = "active_power"
     _attr_device_class: SensorDeviceClass = SensorDeviceClass.POWER
+    _attr_should_poll = True  # BaseZhaEntity defaults to False
     _attr_state_class: SensorStateClass = SensorStateClass.MEASUREMENT
     _unit = POWER_WATT
     _div_mul_prefix = "ac_power"
 
-    @property
-    def should_poll(self) -> bool:
-        """Return True if HA needs to poll for state changes."""
-        return True
-
     @property
     def extra_state_attributes(self) -> dict[str, Any]:
         """Return device state attrs for sensor."""
@@ -305,14 +308,10 @@ class ElectricalMeasurementApparentPower(
 
     SENSOR_ATTR = "apparent_power"
     _attr_device_class: SensorDeviceClass = SensorDeviceClass.APPARENT_POWER
+    _attr_should_poll = False  # Poll indirectly by ElectricalMeasurementSensor
     _unit = POWER_VOLT_AMPERE
     _div_mul_prefix = "ac_power"
 
-    @property
-    def should_poll(self) -> bool:
-        """Poll indirectly by ElectricalMeasurementSensor."""
-        return False
-
 
 @MULTI_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
 class ElectricalMeasurementRMSCurrent(ElectricalMeasurement, id_suffix="rms_current"):
@@ -320,14 +319,10 @@ class ElectricalMeasurementRMSCurrent(ElectricalMeasurement, id_suffix="rms_curr
 
     SENSOR_ATTR = "rms_current"
     _attr_device_class: SensorDeviceClass = SensorDeviceClass.CURRENT
+    _attr_should_poll = False  # Poll indirectly by ElectricalMeasurementSensor
     _unit = ELECTRIC_CURRENT_AMPERE
     _div_mul_prefix = "ac_current"
 
-    @property
-    def should_poll(self) -> bool:
-        """Poll indirectly by ElectricalMeasurementSensor."""
-        return False
-
 
 @MULTI_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
 class ElectricalMeasurementRMSVoltage(ElectricalMeasurement, id_suffix="rms_voltage"):
@@ -335,14 +330,10 @@ class ElectricalMeasurementRMSVoltage(ElectricalMeasurement, id_suffix="rms_volt
 
     SENSOR_ATTR = "rms_voltage"
     _attr_device_class: SensorDeviceClass = SensorDeviceClass.CURRENT
+    _attr_should_poll = False  # Poll indirectly by ElectricalMeasurementSensor
     _unit = ELECTRIC_POTENTIAL_VOLT
     _div_mul_prefix = "ac_voltage"
 
-    @property
-    def should_poll(self) -> bool:
-        """Poll indirectly by ElectricalMeasurementSensor."""
-        return False
-
 
 @MULTI_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
 class ElectricalMeasurementFrequency(ElectricalMeasurement, id_suffix="ac_frequency"):
@@ -350,14 +341,10 @@ class ElectricalMeasurementFrequency(ElectricalMeasurement, id_suffix="ac_freque
 
     SENSOR_ATTR = "ac_frequency"
     _attr_device_class: SensorDeviceClass = SensorDeviceClass.FREQUENCY
+    _attr_should_poll = False  # Poll indirectly by ElectricalMeasurementSensor
     _unit = FREQUENCY_HERTZ
     _div_mul_prefix = "ac_frequency"
 
-    @property
-    def should_poll(self) -> bool:
-        """Poll indirectly by ElectricalMeasurementSensor."""
-        return False
-
 
 @MULTI_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
 class ElectricalMeasurementPowerFactor(ElectricalMeasurement, id_suffix="power_factor"):
@@ -365,13 +352,9 @@ class ElectricalMeasurementPowerFactor(ElectricalMeasurement, id_suffix="power_f
 
     SENSOR_ATTR = "power_factor"
     _attr_device_class: SensorDeviceClass = SensorDeviceClass.POWER_FACTOR
+    _attr_should_poll = False  # Poll indirectly by ElectricalMeasurementSensor
     _unit = PERCENTAGE
 
-    @property
-    def should_poll(self) -> bool:
-        """Poll indirectly by ElectricalMeasurementSensor."""
-        return False
-
 
 @MULTI_MATCH(
     generic_ids=CHANNEL_ST_HUMIDITY_CLUSTER, stop_on_match_group=CHANNEL_HUMIDITY
@@ -472,25 +455,8 @@ class SmartEnergyMetering(Sensor):
 
 @MULTI_MATCH(
     channel_names=CHANNEL_SMARTENERGY_METERING,
-    models={"TS011F"},
     stop_on_match_group=CHANNEL_SMARTENERGY_METERING,
 )
-class PolledSmartEnergyMetering(SmartEnergyMetering):
-    """Polled metering sensor."""
-
-    @property
-    def should_poll(self) -> bool:
-        """Poll the entity for current state."""
-        return True
-
-    async def async_update(self) -> None:
-        """Retrieve latest state."""
-        if not self.available:
-            return
-        await self._channel.async_force_update()
-
-
-@MULTI_MATCH(channel_names=CHANNEL_SMARTENERGY_METERING)
 class SmartEnergySummation(SmartEnergyMetering, id_suffix="summation_delivered"):
     """Smart Energy Metering summation sensor."""
 
@@ -523,6 +489,23 @@ class SmartEnergySummation(SmartEnergyMetering, id_suffix="summation_delivered")
         return round(cooked, 3)
 
 
+@MULTI_MATCH(
+    channel_names=CHANNEL_SMARTENERGY_METERING,
+    models={"TS011F"},
+    stop_on_match_group=CHANNEL_SMARTENERGY_METERING,
+)
+class PolledSmartEnergySummation(SmartEnergySummation):
+    """Polled Smart Energy Metering summation sensor."""
+
+    _attr_should_poll = True  # BaseZhaEntity defaults to False
+
+    async def async_update(self) -> None:
+        """Retrieve latest state."""
+        if not self.available:
+            return
+        await self._channel.async_force_update()
+
+
 @MULTI_MATCH(channel_names=CHANNEL_PRESSURE)
 class Pressure(Sensor):
     """Pressure sensor."""
@@ -638,12 +621,12 @@ class ThermostatHVACAction(Sensor, id_suffix="hvac_action"):
 
     @classmethod
     def create_entity(
-        cls,
+        cls: type[_ThermostatHVACActionSelfT],
         unique_id: str,
         zha_device: ZHADevice,
         channels: list[ZigbeeChannel],
-        **kwargs,
-    ) -> ZhaEntity | None:
+        **kwargs: Any,
+    ) -> _ThermostatHVACActionSelfT | None:
         """Entity Factory.
 
         Return entity if it is a supported configuration, otherwise return None
@@ -760,16 +743,17 @@ class RSSISensor(Sensor, id_suffix="rssi"):
     _attr_device_class: SensorDeviceClass = SensorDeviceClass.SIGNAL_STRENGTH
     _attr_entity_category = EntityCategory.DIAGNOSTIC
     _attr_entity_registry_enabled_default = False
+    _attr_should_poll = True  # BaseZhaEntity defaults to False
     unique_id_suffix: str
 
     @classmethod
     def create_entity(
-        cls,
+        cls: type[_RSSISensorSelfT],
         unique_id: str,
         zha_device: ZHADevice,
         channels: list[ZigbeeChannel],
-        **kwargs,
-    ) -> ZhaEntity | None:
+        **kwargs: Any,
+    ) -> _RSSISensorSelfT | None:
         """Entity Factory.
 
         Return entity if it is a supported configuration, otherwise return None
@@ -784,11 +768,6 @@ class RSSISensor(Sensor, id_suffix="rssi"):
         """Return the state of the entity."""
         return getattr(self._zha_device.device, self.unique_id_suffix)
 
-    @property
-    def should_poll(self) -> bool:
-        """Poll the entity for current state."""
-        return True
-
 
 @MULTI_MATCH(channel_names=CHANNEL_BASIC)
 class LQISensor(RSSISensor, id_suffix="lqi"):
@@ -810,7 +789,7 @@ class TimeLeft(Sensor, id_suffix="time_left"):
     _unit = TIME_MINUTES
 
 
-@MULTI_MATCH(channel_names="ikea_airpurifier", models={"STARKVIND Air purifier"})
+@MULTI_MATCH(channel_names="ikea_airpurifier")
 class IkeaDeviceRunTime(Sensor, id_suffix="device_run_time"):
     """Sensor that displays device run time (in minutes)."""
 
@@ -820,7 +799,7 @@ class IkeaDeviceRunTime(Sensor, id_suffix="device_run_time"):
     _unit = TIME_MINUTES
 
 
-@MULTI_MATCH(channel_names="ikea_airpurifier", models={"STARKVIND Air purifier"})
+@MULTI_MATCH(channel_names="ikea_airpurifier")
 class IkeaFilterRunTime(Sensor, id_suffix="filter_run_time"):
     """Sensor that displays run time of the current filter (in minutes)."""
 
